// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <camera.h>
#include <stage.h>

#include <misc.h>

camera::camera(stage* parent, const spritedata* gfx, int xpos, int ypos)
: guard(parent, gfx, xpos, ypos)
{
	m_fovangle=BIOSDEGREES(35);
	m_fovdepth=100;
	enablerotation(true);
	setradarcolor(S_RADAR_CAM);
	setanimtype(sprite::a_timed);
	enablespritecollision(false);
	setblockssight(false);

	setdescription("A security camera.");
}

//#define PROFILE_CAMERA
//#define DEBUG_CAMERA

#ifdef PROFILE_CAMERA
TOOL_PROFILER_INIT;
#endif

void camera::compute(bool lowtime)
{
#ifdef PROFILE_CAMERA
	TOOL_PROFILER_START;
#endif	
	scriptable::compute(lowtime);

	if(lowtime) return;	// too long to complete in 50 ticks.
	
	sprite** sprites=m_parent->getsprites();
	sprite* sprite;
	int count=m_parent->getspritecount();
	int tmpx, tmpy;
	int dist;
	
	int mycenterx=getboundcenterx();
	int mycentery=getboundcentery();
	for (int i=0; i<count;i++)
	{
		sprite=sprites[i];
		if (sprite==this) continue;	// skip myself
		if (sprite->issightable() && sprite->getcollidemapno()==getcollidemapno())
		{
			tmpx=sprite->getboundcenterx()-mycenterx;
			tmpy=sprite->getboundcentery()-mycentery;
			dist=bios_sqrt(tmpx*tmpx+tmpy*tmpy);
#ifdef DEBUG_CAMERA
			uprintf("dist %d\n", dist);
#endif

			if (dist>m_fovdepth) continue;

#ifdef DEBUG_CAMERA
			uprintf("myangle %d other %d %d (a %d) diff %d fov %d\n", m_angle,tmpx,tmpy,REALDEGREES(bios_atan2(tmpx, tmpy)),(short)(REALDEGREES(bios_atan2(tmpx, tmpy))-m_angle), REALDEGREES(m_fovangle));
#endif
			if (abs((short)(bios_atan2(tmpx, tmpy)-m_angle))>m_fovangle) continue;

#ifdef DEBUG_CAMERA
			uprintf("checking block\n");
#endif
			if (!sightblocked(sprite->getboundxpos(), sprite->getboundypos(),sprite->getboundxsize(), sprite->getboundysize(), sprite, false))	m_parent->setsighted(this,sprite);
		}
	}
#ifdef PROFILE_CAMERA
	TOOL_PROFILER_END;
#endif
}

void camera::push(sprite* pusher)
{
	// A camera won't notice if you run into it.
}



