// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <door.h>
#include <vbl.h>
#include <stage.h>

door::door(stage* parent, const spritedata* gfx, int xpos, int ypos)
: sprite(parent, gfx, xpos, ypos)
{
	m_closedx=xpos;
	m_closedy=ypos;
	m_openx=0;
	m_openy=0;
	m_speed_num=1;
	m_speed_denom=2;
	m_locked=false;
	m_autoclose=300;	// 5 seconds
	m_closeat=-1;
	enablespritecollision(true);
	setradarcolor(2);	// green, same as map structures

	// Base class ctor calls stage::addsprite(), which in turn calls sprite::setcollidemapno() to set the default
	// collide map, not door::setcollidemapno(), resulting in a bad sprite priority for this door. Making
	// door::setcollidemapno() virtual would not help here as at the time stage::addsprite() is called there is no
	// door object yet as this ctor has not yet been called. So we have no choice but to work around this problem
	// by calling the proper implementation here.
	setcollidemapno(getcollidemapno());
}

void door::compute(bool lowtime)
{
	if(m_closeat!=-1 && m_parent->getticks()>=m_closeat)
	{
		close();
		m_closeat=-1;
	}
	sprite::compute(lowtime);
}

void door::action(sprite* actor)
{
	if(!m_locked)
		open();
}

void door::open()
{
	moveto(m_openx, m_openy, m_speed_num, m_speed_denom);
	if(m_autoclose!=-1)
		m_closeat=m_parent->getticks()+m_autoclose;
}

void door::close()
{
	moveto(m_closedx, m_closedy, m_speed_num, m_speed_denom);
	m_closeat=-1;
}

