// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <explosive.h>

explosive::explosive(level* parent, const spritedata* itemgraphics, const spritedata* explodegraphics, int xpos, int ypos)
: item(parent, itemgraphics, xpos, ypos)
{
	m_exploding=0;
	m_exploded=false;
	m_explosionlength=90;	// ticks
	ASSERT(explodegraphics->xsize==64 && explodegraphics->ysize==64, "invalid explosion dimensions");
	m_explosion=new sprite(parent, explodegraphics, xpos, ypos);
	m_explosion->setbounding(b_right, 16);
	m_explosion->setbounding(b_bottom, 16);
	m_explosion->hide();
	setdescription("Explosives.");
}

explosive::~explosive()
{
	// don't delete m_explosion, you don't know if it's still there!
}

void explosive::explode(sprite* who, sprite* onwhat, int explodetime)
{
	setpickable(false);
	m_parent->removefrominventory(this, who);
	if(onwhat)
	{
		drop(onwhat->getboundxpos(), onwhat->getboundypos());
		setobjbefore(onwhat);
	}
	else
		drop(who->getboundxpos(), who->getboundypos());
	m_exploding=1;
	m_explodetime=m_parent->getticks()+explodetime;
}

void explosive::compute(bool lowtime)
{
	item::compute(lowtime);
	if(m_exploding && m_parent->getticks()>m_explodetime)
	{
		if(m_exploding==1)
		{
			// hide item, show explosion
			hide();
			m_explosion->setcollidemapno(getcollidemapno());	// explosion needs to work on the same level as the explosive object
			m_explosion->setpriority(getcollidemapno()>0?getcollidemapno()-1:0);	// make sure the explosion does not cover walls
			m_explosion->setpos(getboundcenterx()-48/2, getboundcentery()-48/2);	// center it over the position of the explosive object
			m_explosion->resetanimation();
			m_explosion->show();
			m_exploded=true;
			m_exploding=2;
			m_explodetime=m_parent->getticks()+m_explosionlength;
			
			// kill nearby sprites
			m_explosion->enablespritecollision(true);
			for(int i=0; i<m_parent->getspritecount();i++)
			{
				if(m_parent->getsprites()[i]==m_explosion) continue;
				if(m_explosion->collides(m_parent->getsprites()[i]))
				{
					m_parent->getsprites()[i]->stop();
					m_parent->getsprites()[i]->hide();
					m_parent->getsprites()[i]->setsightable(false);
					m_parent->getsprites()[i]->setaudible(false);
				}
			}
			// This is necessary because otherwise attracting guard will not work as the explosion occupies their moveto() target.
			m_explosion->enablespritecollision(false);

		}
		else if(m_exploding==2)
		{
			m_explosion->hide();
			m_exploding=0;
		}
	}
}
