// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

// GBA data formats

#ifndef _FORMATS_H
#define _FORMATS_H

#include <gbamappy.h>

/// Map data structure.
typedef struct
{
	int num_bgs;				///< Number of BGs used.
	const u16** mapdata;		///< Raw BG data in 32x32 chunks (optimized for DMA)
	const u8** mapdata_unopt;	///< Raw map data without flipbits in a linear representation
	const BLKSTR* blockstr;		///< Tile flags.
	const u16* colormap;		///< BG palette.
	int colormapsize;			///< Size of BG palette.
	const u8* tileset;			///< Tileset data.
	int tilesetsize;			///< Size of tileset, in shorts.
	int xsize;					///< Horizontal size of map, in 8x8 tiles.
	int ysize;					///< Vertical size of map, in 8x8 tiles.
} mapdata;

/// Sprite graphics data structure.
typedef struct
{
	const u8* graphics;			///< Raw data.
	u32 animations;				///< Number of animations.
	u32 phases;					///< Number of phases per animation.
	u32 eventsperphase;			///< Events (pixels moved or 1/60 seconds) per animation phase.
	u32 xsize;					///< X size, in pixels.
	u32 ysize;					///< Y size, in pixels.
	int col_mode;				///< Color mode (1 == 256 colors, 0 == 16 colors).
	int palette;				///< Palette number, only used for 16 color sprites.
} spritedata;

/// Message box portrait graphics data structure.
typedef struct
{
	const u8* graphics;			///< Raw data.
	u32 xsize;					///< X size, in 8x8 tiles.
	u32 ysize;					///< Y size, in 8x8 tiles.
	const u16* palette;			///< Palette data, 16 entries.
} portraitdata;

/// Backdrop graphics data structure.
typedef struct
{
	const u8* bgtiles;			///< Raw tile data.
	int bgtiles_size;			///< Size of tile data, in shorts.
	const u16* bgpal;			///< Palette data, 240 entries.
	const u16* bgmap;			///< Raw BG map data.
} backdropdata;

/// Font graphics data structure.
typedef struct
{
	const u8* font;				///< Raw font data.
} fontdata;

/// Palette data structure
typedef struct
{
	const u16* palette;			///< Palette data.
	u32 entries;				///< Number of entries.
} palettedata;

#endif
