/* V1.3.12, please see mapwin.htm docs 'Exporting map for GBA' */

#ifndef _GBAMAPPY_H
#define _GBAMAPPY_H

#define AN_END -1			/* Animation types, AN_END = end of anims */
#define AN_NONE 0			/* No anim defined */
#define AN_LOOPF 1		/* Loops from start to end, then jumps to start etc */
#define AN_LOOPR 2		/* As above, but from end to start */
#define AN_ONCE 3			/* Only plays once */
#define AN_ONCEH 4		/* Only plays once, but holds end frame */
#define AN_PPFF 5			/* Ping Pong start-end-start-end-start etc */
#define AN_PPRR 6			/* Ping Pong end-start-end-start-end etc */
#define AN_PPRF 7			/* Used internally by playback */
#define AN_PPFR 8			/* Used internally by playback */
#define AN_ONCES 9		/* Used internally by playback */


/// Structure for data blocks.
 typedef struct {
long int bgoff, 	///< offsets from start of graphic blocks (bg)
	fgoff;			///< offsets from start of graphic blocks (fg)
long int fgoff2,    ///< more overlay blocks
	fgoff3; 		///< more overlay blocks
unsigned long int user1, ///< user long data 1
	user2;	///< user long data 2
unsigned short int user3,	///< user short data 3
	user4;	///< user short data 4
unsigned char user5,	///< user byte data 5
	user6,	///< user byte data 6
	user7;	///< user byte data 7

#if 0	// totally moronic
unsigned char tl : 1;				/* bits for collision detection */
unsigned char tr : 1;
unsigned char bl : 1;
unsigned char br : 1;
unsigned char trigger : 1;			/* bg transparency */
unsigned char unused1 : 1;
unsigned char unused2 : 1;	/* this is the flip-h bit */
unsigned char unused3 : 1;	/* this is the flip-v bit */
#else
/// Miscellaneous flags.
union {
	unsigned char bits;	///< All flags. Needed for compatibility with Mappy export format.
	
	/// Flags as individual bits.
	struct {
		unsigned char tl : 1;				///< Bits for collision detection. Top left.
		unsigned char tr : 1;				///< Bits for collision detection. Top right.
		unsigned char bl : 1;				///< Bits for collision detection. Bottom left.
		unsigned char br : 1;				///< Bits for collision detection. Bottom right.
		unsigned char trigger : 1;			///< Half-high block. Originally BG transparency.
		unsigned char unused1 : 1;	///< Walkable block. Originally unused.
		unsigned char unused2 : 1;	///< Invisible on radar. Originally the flip-h bit (which is unneeded when merging the flipbits).
		unsigned char unused3 : 1;	///< This is the flip-v bit.
	};
};
#endif

} BLKSTR;

// Masks to apply on mapdata_unopt to get info on a certain map position.
#define UNOPT_BLOCK_MASK 0x80		///< Is this block != 0 (empty space)?
#define UNOPT_WALKABLE_MASK 0x01	///< Is this block walkable?
#define UNOPT_INVISIBLE_MASK 0x02	///< Is this block invisible?
#define UNOPT_HALFHIGH_MASK 0x04	///< Is this block half-high?

#if 0	// No BG animation in enjin.
/// Animation control structure.
typedef struct {
signed char antype;	/* Type of anim, AN_? */
signed char andelay;	/* Frames to go before next frame */
signed char ancount;	/* Counter, decs each frame, till 0, then resets to andelay */
signed char anuser;	/* User info */
long int ancuroff;	/* Points to current offset in list */
long int anstartoff;	/* Points to start of blkstr offsets list, AFTER ref. blkstr offset */
long int anendoff;	/* Points to end of blkstr offsets list */
} ANISTR;
#endif

#endif
