// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#ifndef _EXPLOSIVE_H
#define _EXPLOSIVE_H

#include <item.h>

/// This is an explosives item. After explode() is called it will blow up, taking all nearby (i.e. within a 48x48 square around
/// its position), visible and collidable objects with it, i.e. it hides them.
class explosive : public item
{
public:
	/// The explosive constructor.
	/// \param parent The parent level.
	/// \param itemgraphics A set of sprite graphics as used for any item.
	/// \param explodegraphics A 64x64 single-animation/multi-phase set of sprite graphics used for the explosion effect. Only the top left 48x48 rectangle should be used.
	/// \param xpos Explosive's initial position, X coordinate
	/// \param ypos Explosive's initial position, Y coordinate
	explosive(level* parent, const spritedata* itemgraphics, const spritedata* explodegraphics, int xpos, int ypos);

	/// The explosive destructor. Deletes the explosion animation sprite.
	~explosive();

	/// Sets the amount of time the explosion animation will be visible. Defaults to 90 ticks.
	/// \param	length Length of explosion, in ticks.
	void setexplosionlength(int length) { m_explosionlength=length;}
	
	/// Initiate the explosion. The explosive will drop itself either at the position of onwhat, or at the position of who, if onwhat is a NULL
	/// pointer. It will sit there for the given amount of ticks and then blow up, eventually hiding itself. This method is best called from
	/// a custom playerusesitem() implementation.
	/// \param who Who set off the explosive.
	/// \param onwhat Sprite to place the explosive on, or NULL to drop it on the floor.
	/// \param explodetime Delay until the explosive goes off, in ticks.
	void explode(sprite* who, sprite* onwhat, int explodetime);

	/// Performs the explosion.
	virtual void compute(bool lowtime);

	/// This will inform you if the explosives have gone off. It is self-resetting, i.e. it will only return true once per explosion!
	/// \returns Did I explode?
	bool exploded() { bool e=m_exploded; m_exploded=false; return e;}

	/// This will inform you whether the explosion has finished already, i.e. the explosion sprite is no longer visible.
	bool explosionfinished() { return !m_explosion->isvisible();}

private:
	bool m_exploded;		///< Did I explode?
	int m_exploding;		///< Explosion state. 0 == idle, 1 == counting down, 2 == explode now.
	int m_explodetime;		///< Time at which the explosion is supposed to start or to end, depending on m_exploding.
	int m_explosionlength;	///< Time the explosion is visible, in ticks.
	sprite* m_explosion;	///< Explosion animation sprite.
};

#endif
