// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#ifndef _INPUT_H
#define _INPUT_H

#include <mygba.h>
#include <object.h>

/// Provides user input. Call init() before using the static methods. input is a singleton.
class input : private object
{
public:
	static bool action() { return m_action;}		///< \returns Action button pressed?
	static bool info() { return m_info;}			///< \returns Info button pressed?
	static bool selectleft() { return m_sleft;}		///< \returns Select left button pressed?
	static bool selectright() { return m_sright;}	///< \returns Select right button pressed?
	static bool up() { return m_up;}				///< \returns Up pressed?
	static bool down() { return m_down;}			///< \returns Down pressed?
	static bool left() { return m_left;}			///< \returns Left pressed?
	static bool right() { return m_right;}			///< \returns Right pressed?
	static bool select() { return m_select;}		///< \returns Select button pressed?
	static bool start() { return m_start;}			///< \returns Start button pressed?

	/// Register the input with the VBlank handler. The input data will not be updated before this method is called.
	static void init();

	/// Unregister the input with the VBlank handler. The input data will not be updated after this method is called.
	static void uninit();
private:
	/// Updates the input state.
	virtual void vbl_handler();

	/// This is an artefact of not-so-clean design. Only objects can register with the VBlank handler, thus input needs
	/// to be an object, which means it has a position and therefore needs a setpos() method...
	void setpos(int x, int y);

	static input* m_instance;			///< The input instance.

	static volatile bool m_action;		///< Action button state
	static volatile bool m_info;		///< Info button state
	static volatile bool m_sleft;		///< Select left button state
	static volatile bool m_sright;		///< Select right button state
	static volatile bool m_up;			///< Up state
	static volatile bool m_down;		///< Down state
	static volatile bool m_left;		///< Left state
	static volatile bool m_right;		///< Right state
	static volatile bool m_select;		///< Select button state
	static volatile bool m_start;		///< Start button state

	static bool m_sranded;		///< Random number generator initialized?
};

#endif
