// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#ifndef _ITEM_H
#define _ITEM_H

#include <sprite.h>
#include <level.h>

#define I_ITEM_WIDTH 16		///< Fixed width of an item
#define I_ITEM_HEIGHT 16	///< Fixed height of an item

class level;

///	An inventory item. The standard action for this type of object is to get picked up and added to it's parent level's current player's
/// inventory. Also, if collision is enabled for the item, walking over it will pick it up by default.
class item : public sprite
{
public:
	/// The item constructor.
	item(level* parent, const spritedata* gfx, int xpos=0, int ypos=0);
	/// The item destructor. Removes the item from all inventories.
	~item();

	/// Controls the automatic picking up of collidable items by players. If enabled, which is the default, a player walking over an
	/// item with sprite collision enabled will pick up the item.
	/// \param pick Should a player walking into me pick me up?
	void enableautopickup(bool pick) { m_autopick=pick;}

	bool getautopickup() { return m_autopick;}	///< \returns Whether automatic pickup is enabled.

	/// Controls whether this item can be picked up. Defaults to on.
	/// \param pick Can I be picked up?
	void setpickable(bool pick) { m_pickable=pick;}

	bool ispickable() { return m_pickable;}	///< \returns Whether this item can be picked up.

	/// Makes the item invisible for the purposes of collision-detection, sight-blocking etc. and moves it to the given on-screen position.
	/// \param x X position relative to the visible screen.
	/// \param y Y position relative to the visible screen.
	void pickup(int x, int y);

	/// Makes the item visible for purposes of collision-detection, sight-blocking etc. and moves it to the given map position.
	/// This method turns off sprite collision on the dropped item.
	/// \param x X position relative to the map origin.
	/// \param y Y position relative to the map origin.
	void drop(int x, int y);

	/// The standard action for an item is to get picked up, i.e. to add itself to its parent's current player's inventory.
	virtual void action(sprite* actor);

protected:
	level* m_parent;		///< The item's parent level. Items only work with levels, not with generic stages.

private:
	virtual void push(sprite* pusher);

	bool m_autopick;		///< Should a player crashing into me pick me up automatically?
	bool m_pickable;		///< Can this item be picked up?
};

#endif

