// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#ifndef _LEVELPLAYER_H
#define _LEVELPLAYER_H

#include <player.h>
#include <item.h>
#include <level.h>

class item;

#define LP_INVENTORY_SIZE 8		///< Maximum size of a level player's inventory.

/// This is a type of player that supports the level class's inventory system. Manipulate its inventory by using the methods provided there.
/// Just as with its base class you can use multiple instances of levelplayer at a time. Each levelplayer will register and unregister
/// itself with its parent level, but you will have to tell the level which player to use for the purposes of inventory handling.
class levelplayer : public player
{
	friend class level;		// level needs access to the inventory methods

public:
	/// The level player constructor. Registers the player with its parent level.
	levelplayer(level* parent, const spritedata* gfx, int xpos=0, int ypos=0);
	/// The level player destructor. Unregisters the player from its parent level.
	~levelplayer();

	int getinventorysize() { return m_numitems;}	///< \returns The number of items in the player's inventory.
	
	/// Use this to get a certain item in the player's inventory.
	/// \param num Number of the inventory item.
	/// \returns A pointer to the item.
	item* getinventory(int num) { return m_inventory[num];}
	
	/// Use this method to check if this level player tried and was allowed to cross the finish line.
	/// \returns Has this level player crossed the finish line?
	bool hascrossedfinishline() { return m_finishlinepassed;}

	/// Controls whether this level player is allowed to cross the finish line. The default is off.
	/// \param allow Allow this level player to cross the finish line?
	void allowcrossfinishline(bool allow) { m_canpassfinishline=allow;}

private:
	/// Adds an item to my inventory.
	/// \param what An item.
	void addtoinventory(item* what);
	/// Removes an item from my inventory.
	/// \param what An item.
	void removefrominventory(item* what);

	level* m_parent;	///< The level player's parent. Level players only work with levels, not with generic stages.

	item* m_inventory[LP_INVENTORY_SIZE];	///< The player's inventory.
	int m_numitems;							///< Number of items in the player's inventory.

	bool m_finishlinepassed;	///< Has this level player passed the finish line?
	bool m_canpassfinishline;	///< Can this level player pass the finish line?

};

#endif

