// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#ifndef _MISC_H
#define _MISC_H

#include <mygba.h>

/// A transparent tile. Occasionally comes in handy.
extern "C" const unsigned short transparent_tile[];

/// Replacement for the broken ham_SetMapTileEx() function. It misses some functionality, but at least works
/// with 16-color non-CBB-aligned tiles.
void spam_SetMapTileEx(u8 bgno,  u32 x, u32 y, u32 tileno, u32 hflip, u32 vflip, u32 palno);
//#define spam_SetMapTileEx ham_SetMapTileEx

void spam_CopyObjToOAM();
void spam_CreateWin(u8 winno, u8 x1, u8 y1, u8 x2, u8 y2, u8 inside_on_for, u8 outside_on_for, u8 fx_on);
//#define spam_CreateWin ham_CreateWin

void spam_SetObjMode(u8 objno, u8 obj_mode);
//#define spam_SetObjMode ham_SetObjMode
						
short bios_atan2(short x, short y);
u16 bios_sqrt(u32 num);
void bios_lz77uncompvram(u16* src, u16* dest);
void bios_lz77uncompwram(u16* src, u16* dest);

extern int bios_generic_junk;	///< Scratch space for the bios_generic() function

int bios_div(int num, int denom, int* rem=&bios_generic_junk, int* absquot=&bios_generic_junk);

/// A generic BIOS call function. Accepts up to four arguments, returns r0, writes other results to given locations. Parameters default to 0,
/// return locations default to nirvana.
/// \param swi	SWI number
/// \param r0 Argument passed in R0
/// \param r1 Argument passed in R1
/// \param r2 Argument passed in R2
/// \param r3 Argument passed in R3
/// \param rr1 Location for return value in R1
/// \param rr2 Location for return value in R2
/// \param rr3 Location for return value in R3
int bios_generic(u8 swi, int r0=0, int r1=0, int r2=0, int r3=0,
				 int* rr1=&bios_generic_junk, int* rr2=&bios_generic_junk, int* rr3=&bios_generic_junk);

#define SPAM_PROF_GLOBAL static int maxvcnt=0
#define SPAM_PROF_INIT int vcnt=R_VCNT
#define SPAM_PROF_FINISH if(R_VCNT-vcnt>maxvcnt) { maxvcnt=R_VCNT-vcnt; uprintf("%s: max vcnt %d\n",__FILE__, maxvcnt); }

#define MARK uprintf("mark at %d in %s reached\n",__LINE__,__FUNCTION__)

/// Checks if two rectangles overlap.
static inline bool overlap(int x11, int y11, int x12, int y12, int x21, int y21, int x22, int y22)
{
	if((x21>x12)||(x22<x11)||(y22<y11)||(y21>y12))
		return false;
	else
		return true;
}

#define SGN(x) ((x)?(((x)>0)?1:-1):0)

void printmeminfo(void);
void my_new_handler();

#endif

