// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#ifndef _PLAYER_H
#define _PLAYER_H

#include <sprite.h>

/// This is a sprite controlled by the D-Pad and the action button. You can use more than one object of this type at a time. A player sprite
/// autoadjusts, collides, blocks sight, is sightable and has motion-based animation by default.
class player : public sprite
{
public:
	/// The player constructor.
	player(stage* parent, const spritedata* gfx, int xpos=0, int ypos=0);

	/// Freeze this object, i.e. detach it from the input device.
	void freeze() { m_freeze=true;}
	
	/// Unfreeze this object, i.e. attach it to the input device.
	void unfreeze() { m_freeze=false;}

	/// Sets the movement speed used when reacting to user input. Defaults to 2.
	/// \param speed The desired movement speed.
	void setspeed(int speed) { m_speed=speed;}
	
	int getspeed() { return m_speed;}	///< \returns The current movement speed.

	/// A player reacts to user input (unless frozen) in addition to the standard sprite functionality.
	virtual void compute(bool lowtime);

private:
	bool m_ap;		///< "A" button pressed?
	bool m_freeze;	///< Frozen?
	int m_speed;	///< Movement speed.
};

#endif

