// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#ifndef _SWAG_H
#define _SWAG_H

#include <sprite.h>
#include <level.h>

/// The swag class. These sprites just sit there until someone calls their action() method, upon which they hide and add their score to
/// the total stage score.
class swag : public sprite
{
public:
	/// The swag constructor.
	swag(level* parent, const spritedata* gfx, int xpos=0, int ypos=0);

	/// Sets the score for this piece of swag. Defaults to zero.
	/// \param score This swag's score.
	void setscore(int score) { m_score=score; }

	/// Returns whether an action has been sucessfully performed on this object.
	/// \returns Whether the swag has been snatched.
	bool issnatched() { return m_snatched; }

	/// The default action for a swag object is to hide() itself and add its score to the stage's total.
	virtual void action(sprite* actor);

	/// Consider myself picked up if someone who is a registered player walks into me.
	virtual void push(sprite* pusher);

	/// Controls whether a collidable swag object gets picked up automatically. Off by default. For this to work the object's sprite
	/// collision properties need to be set up properly. See setspritecollision(), setcollidemapno().
	/// \param apick Enable automatic pickup?
	void enableautopickup(bool apick) { if(!spritecollisionenabled()) enablespritecollision(apick); m_autopickup=apick;}

private:
	level* m_parent;	///< The parent level.
	int m_score;		///< Score for this piece of swag.
	bool m_snatched;	///< Snatched yet?
	bool m_autopickup;	///< Autopickup on push()?
};

#endif

