// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <level.h>
#include <messagebox.h>
#include <vbl.h>
#include <input.h>

#include <stdio.h>	// sprintf()

extern const spritedata frame_sprite;
extern const fontdata console_font;
extern const palettedata msgbox_palette;

extern const spritedata score_sprite;
extern const spritedata marker_sprite;
extern const palettedata sprite16_palette;

level::level(int first_bg, const mapdata* playfield)
: stage(first_bg, playfield)
{
#ifdef DEBUG_LEVELS
	if(input::select())	m_debugenabled=true;
	else m_debugenabled=false;
#endif

	widget::setcharacters(&console_font);
	widget::setpalette(&msgbox_palette);

	m_gotsighted=false;

	m_player=-1;
	m_numplayers=0;

	m_itemselected=-1;
	m_droppingenabled=false;

	m_marker=new sprite(this, &marker_sprite, 256, 0);
	m_marker->setframeofreferencetoscreen(true);
	m_marker->setcollidemapno(getnumlayers()-1);	// brings the item to the front
	m_marker->setdescription(NULL);

	m_score=m_secrets=0;

	m_flx1=m_fly1=m_flx2=m_fly2=0;

	ASSERT(score_sprite.xsize==L_SCORE_NUMBER_XSIZE && score_sprite.ysize==L_SCORE_NUMBER_YSIZE, "invalid score dimensions");
	for(int i=0; i<L_SCORE_NUMBERS; i++)
	{
		m_scorenumber[i]=new sprite(this, &score_sprite,
									SCREEN_WIDTH-L_SCORE_NUMBERS*8-4+i*8, 4);	// position
		m_scorenumber[i]->setanimation(10);	// empty
		m_scorenumber[i]->setframeofreferencetoscreen(true);
		m_scorenumber[i]->setdescription(0);
		m_scorenumber[i]->setpriority(P_ONTOP);
	}
	for(int i=0; i<4; i++)
	{
		m_clocknumber[i]=new sprite(this, &score_sprite,
									SCREEN_WIDTH-L_SCORE_NUMBERS*8-70+i*8+(i>1?4:0), 4);
		if(i==0) m_clocknumber[i]->setanimation(10);
		else m_clocknumber[i]->setanimation(0);
		m_clocknumber[i]->setframeofreferencetoscreen(true);
		m_clocknumber[i]->setdescription(0);
		m_clocknumber[i]->setpriority(P_ONTOP);
		m_clock[i]=0;
	}
	sprite::setpalette16(15, &sprite16_palette);
	modifyscore(0);		// initialize

}

level::~level()
{
	m_marker=0;		// make sure not to dereference marker during destruction

	// some objects (notably those derived from item) use level class interfaces in their dtor, so we need to delete
	// them here where we can be sure those interfaces are still there.
	M_DISCNT_FRCBLK_ENA;	// CONSIDERME: Is there a better way to hide this shebang than blanking the whole screen?
	deleteallsprites();
	M_DISCNT_FRCBLK_DIS;
}

level::state level::endofstage()
{
#ifdef DEBUG_LEVELS
	if(m_debugenabled && input::start())
		return c_abort;
#endif
	sprite* s=getsighted();
	if(s)
	{
		enableradar(false);		// switches of SFX, thus we need to do this before adding the highlights
		addhighlightsprite(s);
		addhighlightsprite(getsightedby());	// highlights will be deleted by stage destructor, no need to care

		// center between the two sprites
		int x=s->getboundcenterx()+(getsightedby()->getboundcenterx()-s->getboundcenterx())/2 - SCREEN_WIDTH/2;
		int y=s->getboundcentery()+(getsightedby()->getboundcentery()-s->getboundcentery())/2 - SCREEN_HEIGHT/2;
		fixpos(&x, &y);
		moveto(x, y, 1, 2);

		m_gotsighted=true;
	}
	if(m_gotsighted)
		return c_lost;

	return c_undefined;
}

void level::sfxlost()
{
	enableradar(false);
	messagebox* m=new messagebox(false, 0, "BUSTED!", NULL, NULL, 7, 2, 240/2-4*8, 160/2-1*8);
	m->writenextpage();
	for(int i=0;i<180/VBL_FRAMESKIP;i++)   VBL_NEXT_FRAME;	// wait 3 seconds
	delete m;
}

void level::sfxwon()
{
	enableradar(false);
	messagebox* m=new messagebox(false, 0, "YOU WIN!", NULL, NULL, 8, 2, 240/2-4*8, 160/2-1*8);
	m->writenextpage();
	for(int i=0;i<180/VBL_FRAMESKIP;i++)   VBL_NEXT_FRAME;	// wait 3 seconds
	delete m;
}

void level::sfxdraw()
{
	enableradar(false);
	messagebox* m=new messagebox(false, 0, "DRAW!", NULL, NULL, 8, 2, 240/2-4*8, 160/2-1*8);
	m->writenextpage();
	for(int i=0;i<180/VBL_FRAMESKIP;i++)   VBL_NEXT_FRAME;	// wait 3 seconds
	delete m;
}

//#define DEBUG_INVENTORY
//#define PROFILE_CLOCK

#ifdef PROFILE_CLOCK
TOOL_PROFILER_INIT;
#endif

void level::compute(bool lowtime)
{
	ASSERT(m_player!=-1, "no player set");

#ifdef DEBUG_LEVELS
	if(m_debugenabled)
	{
		if(input::left())
			setpos(m_xpos-2*VBL_FRAMESKIP, m_ypos);
		if(input::right())
			setpos(m_xpos+2*VBL_FRAMESKIP, m_ypos);
		if(input::down())
			setpos(m_xpos, m_ypos+2*VBL_FRAMESKIP);
		if(input::up())
			setpos(m_xpos, m_ypos-2*VBL_FRAMESKIP);
	}
#endif

	// m_xp avoid auto-repeat
	if(!input::info()) m_bp=false;
	if(!input::selectleft())   m_lp=false;

	if(input::info() && !m_bp)
	{
		m_bp=true;
		if(m_droppingenabled && m_itemselected!=-1)
			removefrominventory(m_players[m_player]->getinventory(m_itemselected), m_players[m_player]);
		else
		{
			while(input::info());
			infoscreen();
		}
	}

	// select next item to the right or first item if none selected, or none if last item selected
	if(input::selectleft() && !m_lp)
	{
		m_lp=true;
		int is=m_players[m_player]->getinventorysize();
		if(is)
		{
			if(m_itemselected==-1) m_itemselected=0;
			else if(m_itemselected<is-1) m_itemselected++;
			else m_itemselected=-1;
			setitemmarker();
		}
	}

#ifdef DEBUG_INVENTORY
	uprintf("itsel %d\n", m_itemselected);
#endif

#ifdef PROFILE_CLOCK
	TOOL_PROFILER_START;
#endif
	m_clock[0]++;
	if(m_clock[0]>(60/VBL_FRAMESKIP)-1)
	{
		m_clock[0]=0;
		m_clock[1]++;
		if(m_clock[1]>59)
		{
			m_clock[1]=0;
			m_clock[2]++;
			if(m_clock[2]>59)
			{
				m_clock[2]=0;
				m_clock[3]++;
			}
		}

		int nums[4];
		nums[0]=bios_div(m_clock[2], 10, &nums[1]); if(nums[0]==0) nums[0]=10;
		nums[2]=bios_div(m_clock[1], 10, &nums[3]);
		for(int i=0;i<4;i++)
			m_clocknumber[i]->setanimation(nums[i]);
	}
#ifdef PROFILE_CLOCK
	TOOL_PROFILER_END;
#endif

	stage::compute(lowtime);
}

#define L_INFOSCREEN_XSIZE 23
#define L_INFOSCREEN_YSIZE 18

void level::infoscreen()
{
	// remember current stage position
	int currentx=m_xpos;
	int currenty=m_ypos;

	// remember radar state, then switch it off
	bool radarstate=radarenabled();
	enableradar(false);

#define CURSPRITE getsprites()[s]
#define SPRITES getsprites()

	int s=-1;	// current sprite number
	int sx=m_players[m_player]->getboundxpos();
	int sy=m_players[m_player]->getboundypos();
	int mindist=0x7fffffff;

	// skip to the closest sprite we can see
	for(int siter=0;siter<getspritecount();siter++)
	{
		if(SPRITES[siter]==m_players[m_player] || !SPRITES[siter]->getdescription() || !SPRITES[siter]->isvisible()) continue;
		int dist=abs(sx-SPRITES[siter]->getboundxpos())+abs(sy-SPRITES[siter]->getboundypos());
		if(dist<mindist)
		{
			mindist=dist;
			s=siter;
		}
	}
	ASSERT(s>-1, "no examinable sprite found");

	messagebox* m=0;

	bool quit=false;
	while(!quit)
	{
		// decide whether to place the highlight at the left or right end of the screen
		bool displayleft=false;
		if(CURSPRITE->getboundxpos()-SCREEN_WIDTH+24<getlexcess()) displayleft=true;

		addhighlightsprite(CURSPRITE, displayleft?24:SCREEN_WIDTH-24);

		// show the description
		if(!m)
			m=new messagebox(false, 0, getsprites()[s]->getdescription(),&frame_sprite,NULL,
							 L_INFOSCREEN_XSIZE,L_INFOSCREEN_YSIZE,displayleft?SCREEN_WIDTH-8-L_INFOSCREEN_XSIZE*8:8,8,1);
		else
		{
			m->settext(CURSPRITE->getdescription());
			m->moveto(displayleft?SCREEN_WIDTH-8-L_INFOSCREEN_XSIZE*8:8,8,2,1);
		}

		m->writenextpage();

		int olds=s;
		sx=CURSPRITE->getboundxpos();
		sy=CURSPRITE->getboundypos();
		while(s==olds)
		{
			bool l=input::left();
			bool r=input::right();
			bool u=input::up();
			bool d=input::down();
			int minx=0x7fffffff;
			int miny=0x7fffffff;
			if(l || r || u || d)	// change sprite highlighted
			{
				// loop over all sprites trying to find the nearest one in the chosen direction
				for(int siter=0;siter<getspritecount();siter++)
				{
					// skip the current sprite, sprites without description and invisible sprites
					if(siter==olds || !SPRITES[siter]->getdescription() || !SPRITES[siter]->isvisible()) continue;

					// get the distance between the sprite and the currently highlighted one
					int dx=sx-SPRITES[siter]->getboundxpos();
					int dy=sy-SPRITES[siter]->getboundypos();

					// skip if it's in the other direction
					if(l && dx<=0) continue;
					if(r && dx>0) continue;
					if(u && dy<=0) continue;
					if(d && dy>0) continue;

					//uprintf("%d is at dx %d dy %d rel to %d\n", siter, dx, dy, olds);

					// skip sprites outside a 90 degree opening angle
					dx=abs(dx);
					dy=abs(dy);
					if((l||r) && dy>dx)	continue;
					if((u||d) && dx>dy)	continue;

					// is this the nearest?
					if( ((l||r) && (dx<minx || (dx==minx && dy<miny))) ||
						((u||d) && (dy<miny || (dy==miny && dx<minx))) )
					{
						minx=dx;
						miny=dy;
						s=siter;
					}
				}
			}

			if(input::info())
			{
				quit=true;
				break;
			}
		}
		hidehighlights();
	}
	if(m) delete m;
	moveto(currentx, currenty, 3, 1);
	while(input::info());
	enableradar(radarstate);
}

bool level::isplayer(sprite* who)
{
	for(int i=0; i<m_numplayers; i++)
		if(m_players[i]==who) return true;
	return false;
}

void level::addtoinventory(item* what, sprite* whos)
{
	ASSERT(m_numplayers>0, "no players registered");
	if(whos==0)	whos=m_players[m_player];
	else if(!isplayer(whos)) return;
	if(what->ispickable())
	{
		m_players[m_player]->addtoinventory(what);
		drawinventory();
	}
}

void level::removefrominventory(item* what, sprite* whos)
{
	if(whos==0)	whos=m_players[m_player];
	else if(!isplayer(whos)) return;
	((levelplayer*)whos)->removefrominventory(what);
	what->drop(whos->getxpos(), whos->getypos());
	// adjust selection
	if(m_itemselected>=((levelplayer*)whos)->getinventorysize())	// sets to -1 after last item dropped
	{
		m_itemselected--;
		setitemmarker();
	}
	drawinventory();
}

void level::removefromallinventories(item* what)
{
	for(int i=0; i<m_numplayers; i++)
	{
		removefrominventory(what, m_players[i]);
	}
}

void level::drawinventory()
{
#ifdef DEBUG_INVENTORY
	uprintf("drawing %d items\n", m_players[m_player]->getinventorysize());
#endif
	for(int i=0; i<m_players[m_player]->getinventorysize();i++)
		m_players[m_player]->getinventory(i)->pickup(4+i*(I_ITEM_WIDTH+4), 4);

	setitemmarker();
}

void level::hideinventory()
{
	for(int i=0; i<m_players[m_player]->getinventorysize();i++)
		m_players[m_player]->getinventory(i)->pickup(256, 0);
	m_itemselected=-1;
	setitemmarker();
}

void level::setitemmarker()
{
	if(m_marker)	// This could be called during destruction, so we need to check if we still have our marker.
	{
		if(m_itemselected==-1)
			m_marker->setpos(256, 0);	// off screen
		else
			m_marker->setpos(4+m_itemselected*(I_ITEM_WIDTH+4), 4);
	}
}

item* level::getitemselected() {
	if(m_itemselected==-1) return NULL;
	else return m_players[m_player]->getinventory(m_itemselected);
}

void level::setitemselected(int itemno)
{
	ASSERT(itemno>=-1 && itemno<m_players[m_player]->getinventorysize(), "invalid item number");
	m_itemselected=itemno;
}

void level::setplayer(levelplayer* player)
{
	for(int i=0; i<m_numplayers;i++)
		if(m_players[i]==player)
		{
			hideinventory();
			m_player=i;
			drawinventory();
			return;
		}
	ASSERT(0, "cannot set unregged player");
}

void level::addplayer(levelplayer* player)
{
	ASSERT(!isplayer(player), "cannot register player twice");
	ASSERT(m_numplayers<L_MAX_PLAYERS-1, "max. num of players exceeded");
	m_players[m_numplayers++]=player;

#ifdef DEBUG_LEVELS
	if(m_debugenabled) player->freeze();
#endif

	// current player defaults to first registered
	if(m_player==-1 && m_numplayers==1)	m_player=0;
}

void level::removeplayer(levelplayer* player)
{
	ASSERT(isplayer(player), "cannot remove unregged player");
	for(int i=0;i<m_numplayers;i++)
	{
		if(m_players[i]==player)
		{
			m_numplayers--;
			if(m_player==i)	m_player=-1;
			for(; i<m_numplayers;i++)
			{
				m_players[i]=m_players[i+1];
			}
			return;
		}
	}
}

void level::action(sprite* me, sprite* them)
{
	if(m_itemselected!=-1 && isplayer(me))
	{
		if(playerusesitem((levelplayer*)me, getitemselected(), them))
		{
			// unselect if used successfully
			m_itemselected=-1;
			setitemmarker();
		}
	}
	else
		if(them) stage::action(me, them);
}

//#define USE_SPRINTF

void level::modifyscore(int delta)
{
	m_score+=delta;

#ifdef USE_SPRINTF

	// the following code needs a whopping three (3) scanlines!
	char s[7];
	sprintf(s, L_SCORE_FORMAT, m_score);
	for(int i=0; i<L_SCORE_NUMBERS; i++)
	{
		m_scorenumber[i]->setanimation(s[i]==32?10:s[i]-'0');
	}
#else
	// this only takes one scanline at worst
	int s[7];
	int sc=m_score;
	for(int i=L_SCORE_NUMBERS;i;i--)
	{
		if(!sc)	s[i-1]=0;
		else sc=bios_div(sc, 10, &s[i-1]);
	}

	bool display_zeroes=false;
	for(int i=0; i<L_SCORE_NUMBERS;i++)
	{
		if(i==L_SCORE_NUMBERS-1) display_zeroes=true;
		if(s[i]) display_zeroes=true;
		m_scorenumber[i]->setanimation(s[i]?s[i]:(display_zeroes?0:10));
	}
#endif	
}

void level::setscoreposition(int x, int y)
{
	for(int i=0; i<L_SCORE_NUMBERS; i++)
		m_scorenumber[i]->setpos(x+L_SCORE_NUMBER_XSIZE*i, y);
}

void level::showscore()
{
	for(int i=0; i<L_SCORE_NUMBERS; i++)
		m_scorenumber[i]->show();
}

void level::hidescore()
{
	for(int i=0; i<L_SCORE_NUMBERS; i++)
		m_scorenumber[i]->hide();
}

void level::setspritepalette(const palettedata* palette)
{
	ASSERT(palette->entries<=240, "sprite palette too large for level");
	stage::setspritepalette(palette);
}

//#define PROFILE_SPRITEMOVED

#ifdef PROFILE_SPRITEMOVED
TOOL_PROFILER_INIT;
#endif

u32 level::spritemoved(sprite* obj, int oldx, int oldy)
{
#ifdef PROFILE_SPRITEMOVED
	TOOL_PROFILER_START;
#endif
	if(isplayer(obj))
	{
		levelplayer* lp=(levelplayer*)obj;

		if(!lp->m_finishlinepassed)
		{
			int x=lp->getboundxpos();
			int y=lp->getboundypos();
			if(overlap(x, y, x+lp->getboundxsize(), y+lp->getboundysize(), m_flx1, m_fly1, m_flx2, m_fly2))
			{
				if(!lp->m_canpassfinishline)
					return(oldy<<16)+oldx;
				else
					lp->m_finishlinepassed=true;
			}
		}
	}
#ifdef PROFILE_SPRITEMOVED
	TOOL_PROFILER_END;
#endif
	return stage::spritemoved(obj, oldx, oldy);
}
