// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <sys/stat.h>
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>

#include "leveled.h"

// This code contains numerous memleaks, but who cares, it will only run in the build environment.

#define SPRITENAME(x) (strlen(objs[x].m_spritename)>0?objs[x].m_spritename:stripbrackets(strdup(objs[x].m_identifier)))

void dumpscript(FILE* f, object* o, object* objs)
{
	fprintf(f, "static const int %s_data[]={ ",stripbrackets(strdup(o->m_identifier)));	// leaking again...
	int j;
	for(j=0; j<o->m_datasize; j++) fprintf(f, "%d, ", o->m_data[j]);
	if(o->m_data[j-1]!=s_enddata) fprintf(f, "%d", s_enddata);	// add s_enddata if not there
	fprintf(f, "};\n");
}

void stdinit(FILE* ini, FILE* gfxdecl, object* obj, char* cls, object* objs, int objnum)
{
	if(obj->INIT_GENERIC_HAS_LUASCRIPT && obj->m_datasize)
	{
		dumpscript(ini, obj, objs);
	}
	{
		char spritename[40];
		strcpy(spritename, obj->m_identifier);
		stripbrackets(spritename);
		if(strlen(obj->m_spritename)>0)	strcpy(spritename, obj->m_spritename);
		fprintf(ini, "%s=new %s(this, &%s_sprite, %d, %d);\n",obj->m_identifier,cls, spritename, obj->m_xpos,obj->m_ypos);
		fprintf(ini, "setstaticsprite(%s, %d);\n", obj->m_identifier, objnum);
		if(obj->INIT_GENERIC_ANIMDIR) fprintf(ini, "%s->setanimdirection(%d);\n", obj->m_identifier, obj->INIT_GENERIC_ANIMDIR);
		fprintf(gfxdecl, "extern const spritedata %s_sprite;\n", spritename);
	}
	if(strlen(obj->m_description)>0)
	{
		fprintf(ini, "%s", obj->m_identifier);
		if(!strcmp(obj->m_description,"none"))
			fprintf(ini, "->setdescription(0);\n");
		else
			fprintf(ini, "->setdescription(\"%s\");\n",escapequotes(obj->m_description));
	}
	if(obj->INIT_GENERIC_INVISIBLE)
		fprintf(ini, "%s->hide();\n", obj->m_identifier);
	if(obj->INIT_GENERIC_HAS_LUASCRIPT)
	{
		fprintf(ini,"%s->startscript(%s_script,sizeof(%s_script)",obj->m_identifier,stripbrackets(strdup(obj->m_identifier)),stripbrackets(strdup(obj->m_identifier)));
		if(obj->m_datasize)	fprintf(ini, ",%s_data", stripbrackets(strdup(obj->m_identifier)));
		fprintf(ini, ");\n");
	}
}

int main(int argc, char** argv)
{
	char outname[strlen(argv[1])+1];
	strncpy(outname, argv[1], strlen(argv[1])-4); outname[strlen(argv[1])-4]=0;
	char lvlname[strlen(outname)+1];
	strcpy(lvlname,outname);
	char* levelname;
	if(strstr(lvlname,"/")) levelname=strstr(lvlname, "/")+1;
	else levelname=lvlname;
	char declname[strlen(outname)+20];
	strcpy(declname, outname);
	strcat(declname, "_lvl_decl.h");
	char ctorname[strlen(outname)+20];
	strcpy(ctorname, outname);
	strcat(ctorname, "_lvl_ctor.h");
	char gfxdeclname[strlen(outname)+20];
	strcpy(gfxdeclname, outname);
	strcat(gfxdeclname, "_lvl_gfx.h");

	strcat(outname, ".lvl");

	int objnum,tmp;
	object objs[MAX_OBJECTS];

	struct stat s;
	assert(stat(outname,&s)==0);

	FILE* in=fopen(outname, "r");
	fread(&objnum, sizeof(int), 1, in);
	fread(&tmp, sizeof(int), 1, in);
	assert(tmp==sizeof(object));
	for(int i=0;i<objnum;i++)
		fread(&objs[i], sizeof(object), 1, in);
	fclose(in);

	FILE* decl=fopen(declname, "w");
	fprintf(decl, "// members for %s. This file has been generated automatically. Do not edit!\n",argv[1]);
	int lcnt,gcnt,scnt,icnt,ecnt,spcnt,ccnt,dcnt,sccnt;
	lcnt=gcnt=scnt=icnt=ecnt=spcnt=ccnt=dcnt=sccnt=0;

	char freeformarray[20][2][40];
	int freeformcnt[20]={0};
	freeformarray[0][0][0]=0;

	for(int i=0;i<objnum;i++)
	{
		// write declarations
		switch(objs[i].m_type)
		{
#define DECLOBJ(type, class) case type: \
	{ \
		if(!strstr(objs[i].m_identifier, "[")) \
			fprintf(decl, #class "* %s;\n", objs[i].m_identifier); \
		else { \
			int f=0; \
			while(freeformarray[f][0][0]!=0) \
			{ \
				if(!strncmp(objs[i].m_identifier,freeformarray[f][0],strstr(objs[i].m_identifier,"[")-objs[i].m_identifier)) { \
					freeformcnt[f]++; break; } \
				f++; \
			} \
			if(freeformarray[f][0][0]==0) { \
				strncpy(freeformarray[f][0],objs[i].m_identifier,strstr(objs[i].m_identifier,"[")-objs[i].m_identifier); \
				freeformarray[f][0][strstr(objs[i].m_identifier,"[")-objs[i].m_identifier]=0; \
				strcpy(freeformarray[f][1], #class); \
				freeformcnt[f]=1; \
			} \
		} \
	} \
	break;

		DECLOBJ(t_levelplayer, levelplayer);
		DECLOBJ(t_guard, guard);
		DECLOBJ(t_swag, swag);
		DECLOBJ(t_item, item);
		DECLOBJ(t_explosive, explosive);
		DECLOBJ(t_sprite, sprite);
		DECLOBJ(t_camera, camera);
		DECLOBJ(t_door, door);
		DECLOBJ(t_scriptable, scriptable);
		case t_finish:	// pseudo-object
			break;
		default:
			abort();
		}
	}

	int f=0;
	while(freeformarray[f][0][0]!=0)
	{
		fprintf(decl, "#define NUM_%s_%s %d\n",levelname,freeformarray[f][0],freeformcnt[f]);
		fprintf(decl, "%s* %s[%d];\n",freeformarray[f][1],freeformarray[f][0],freeformcnt[f]);
		f++;
	}

	fclose(decl);

	FILE* ini=fopen(ctorname, "w");
	fprintf(ini, "// initialization of members for %s. This file has been generated automatically. Do not edit!\n", argv[1]);

	FILE* gfxdecl=fopen(gfxdeclname, "w");
	fprintf(gfxdecl, "// declaration of graphics for %s. This file has been generated automatically. Do not edit!\n", argv[1]);
	bool havescripts=false;
	for(int i=0;i<objnum;i++) if(objs[i].INIT_GENERIC_HAS_LUASCRIPT) havescripts=true;
	if(havescripts)	fprintf(gfxdecl, "#include \"scripts.lua.h\"\n");

	lcnt=gcnt=scnt=icnt=ecnt=spcnt=ccnt=dcnt=sccnt=0;
	for(int i=0;i<objnum;i++)
	{
		// write initialization code
		switch(objs[i].m_type)
		{

#define STDINI(class) stdinit(ini, gfxdecl, &objs[i], #class, objs, i)

		case t_levelplayer:
			STDINI(levelplayer);
			fprintf(ini, "scrolllock(%s);\n",objs[i].m_identifier);
			break;
		case t_guard:
			STDINI(guard);
			fprintf(ini,"%s->setmovetype(sprite::t_astar);\n",objs[i].m_identifier);
			break;
		case t_swag:
			STDINI(swag);
			fprintf(ini, "%s->setscore(%d);\n",objs[i].m_identifier,objs[i].INIT_SWAG_SCORE);
			break;
		case t_item:
			STDINI(item); break;
		case t_explosive:
			if(objs[i].m_datasize)
			{
				dumpscript(ini,&objs[i],objs);
			}
			fprintf(ini, "%s=new explosive(this, &%s_sprite, &explosion_sprite, %d, %d);\n", objs[i].m_identifier, SPRITENAME(i), objs[i].m_xpos, objs[i].m_ypos);
			if(objs[i].INIT_GENERIC_ANIMDIR) fprintf(ini, "%s->setanimdirection(%d);\n", objs[i].m_identifier, objs[i].INIT_GENERIC_ANIMDIR);
			fprintf(gfxdecl, "extern const spritedata %s_sprite;\n",SPRITENAME(i));
			fprintf(gfxdecl, "extern const spritedata explosion_sprite;\n");
			ecnt++;
			break;
		case t_sprite:
			STDINI(sprite);
			fprintf(ini, "%s->enablespritecollision(%s);\n",objs[i].m_identifier,objs[i].INIT_SPRITE_COLLISION?"true":"false");
			fprintf(ini, "%s->enablemapcollision(%s);\n",objs[i].m_identifier,objs[i].INIT_SPRITE_MAPCOLLISION?"true":"false");
			fprintf(ini, "%s->setshovable(%s);\n",objs[i].m_identifier,objs[i].INIT_SPRITE_SHOVABLE?"true":"false");
			fprintf(ini, "%s->setradarcolor(7);\n", objs[i].m_identifier);
			break;
		case t_camera:
			STDINI(camera);
			fprintf(ini, "%s->setangle(%d);\n", objs[i].m_identifier, objs[i].INIT_CAM_ANGLE);
			break;
		case t_door:
			STDINI(door);
			fprintf(ini, "%s->setopenposition(%d,%d);\n",objs[i].m_identifier,objs[i].INIT_DOOR_OPENPOSX,objs[i].INIT_DOOR_OPENPOSY);
			break;
		case t_scriptable:
			STDINI(scriptable); break;
		case t_finish:
			fprintf(ini, "setfinishline(%d, %d, %d, %d);\n", objs[i].m_xpos, objs[i].m_ypos, objs[i].INIT_FINISH_ENDPOSX, objs[i].INIT_FINISH_ENDPOSY);
			break;
		default:
			abort();
		}
	}

}
