// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include "leveled.h"
#include <stdarg.h>
#include <math.h>
#include <stdlib.h>

SDL_Surface* font;
SDL_Surface* typegfx[t_finish];
SDL_Surface* marker;

char* tgfx[] = { "../gfx/farisedit.bmp", "../gfx/galufedit.bmp", "../gfx/gold.bmp", "../gfx/key.bmp", "../gfx/bomb.bmp", "../gfx/picture3.bmp",
	"../gfx/camera1.bmp", "../gfx/wall.bmp", "../gfx/picture1.bmp"};

const char* direction_name[] =
{
	"d_left", "d_up", "d_right", "d_down", "d_upleft", "d_upright", "d_downleft", "d_downright", "d_undefined"
};

#define max(x,y) ((x)>(y)?(x):(y))

#define ARROW_LENGTH 7
#define ARROW_WIDTH 5

void drawline(SDL_Surface* scr, int x1, int y1, int x2, int y2, bool arrow)
{
	Uint16* pix=(Uint16*)(scr->pixels);
	int step=max(abs(x1-x2), abs(y1-y2));
	for(int k=0;k<step;k++)
	{
		int ox=(int)(x1+((double)x2-x1)/step*k);
		int oy=(int)(y1+((double)y2-y1)/step*k);
		pix[ox+oy*scr->w]=0xffff;
	}
	if(arrow && step>0)
	{
		int dx=x2-x1;
		int dy=y2-y1;
		double len=sqrt(dx*dx+dy*dy);
		double slop=(double)dy/dx;
		if(!slop)
		{
			drawline(scr, x1+dx/2, y1+dy/2, (int)(x1+dx/2-dx/len*ARROW_LENGTH), (int)(y1+dy/2-dy/len*ARROW_LENGTH+ARROW_WIDTH),false);
			drawline(scr, x1+dx/2, y1+dy/2, (int)(x1+dx/2-dx/len*ARROW_LENGTH), (int)(y1+dy/2-dy/len*ARROW_LENGTH-ARROW_WIDTH),false);
		}
		else
		{
			double newslop=-(1/slop);
			double a=sqrt(ARROW_WIDTH*ARROW_WIDTH/(1+newslop*newslop));
			drawline(scr, x1+dx/2, y1+dy/2, (int)(x1+dx/2-(dx/len*ARROW_LENGTH)-a), (int)(y1+dy/2-(dy/len*ARROW_LENGTH)-a*newslop),false);
			drawline(scr, x1+dx/2, y1+dy/2, (int)(x1+dx/2-(dx/len*ARROW_LENGTH)+a), (int)(y1+dy/2-(dy/len*ARROW_LENGTH)+a*newslop), false);
		}
	}
}

void drawtext(SDL_Surface* scr, int x, int y, char* fmt, ...)
{
	char buf[256];
	va_list arglist;
	va_start(arglist,fmt);
	vsprintf(buf, fmt, arglist);
	va_end(arglist);
	SDL_Rect src;
	src.w=FONT_WIDTH;src.h=FONT_HEIGHT;
	SDL_Rect tgt={x,y};
	char* b=buf;
	while(*b)
	{
		src.x=((*b-32)&15)*FONT_WIDTH;
		src.y=((*b-32)/16)*16;
		SDL_BlitSurface(font, &src, scr, &tgt);
		tgt.x+=FONT_WIDTH;
		b++;
	}
}

void redraw(SDL_Surface* scr, SDLMappy* map, object* objs, int objnum, int objsel, int scriptsel, bool locktogrid, bool enteri, bool enterd, bool enters)
{
	SDL_FillRect(scr, NULL, SDL_MapRGB(scr->format,0,0,0));
	for(int i=0;i<8;i++)
	{
		map->MapChangeLayer(i);
		map->MapDrawBGT(scr);
	}
	for(int i=0;i<objnum;i++)
	{
		object* o=&objs[i];
		SDL_Rect r={o->m_xpos,o->m_ypos};
		if(o->m_type != t_finish) SDL_BlitSurface(typegfx[o->m_type], NULL, scr, &r);
		else
		{
			drawline(scr, o->m_xpos, o->m_ypos, o->INIT_FINISH_ENDPOSX, o->INIT_FINISH_ENDPOSY);
		}
		int oldx=o->m_xpos;
		int oldy=o->m_ypos;
		int tokencnt=0;
		for(int j=0;j<o->m_datasize;j++)
		{
			//printf("script token %d\n", o->m_scr[j]);
			int x,y;
			SDL_Rect r;
			switch(o->m_data[j])
			{
			case s_waypoint:
				// position
				x=o->m_data[j+1];
				y=o->m_data[j+2];
				if(i==objsel) drawtext(scr, map->GetMapWidth(), tokencnt*FONT_HEIGHT, "wayp %d, %d",x,y);
				drawline(scr, oldx+OBJECT_SIZE/2, oldy+OBJECT_SIZE/2, x+OBJECT_SIZE/2, y+OBJECT_SIZE/2);
				r.x=x;r.y=y;
				SDL_BlitSurface(typegfx[o->m_type], NULL, scr, &r);
				oldx=x;oldy=y;
				break;
			case s_sprite:
				if(i==objsel)
					drawtext(scr, map->GetMapWidth(), tokencnt*FONT_HEIGHT, "sprt %s", o->m_data[j+1]==-1?"UNDEFINED":objs[o->m_data[j+1]].m_identifier);
				break;
			default:
				if(i==objsel)
					drawtext(scr, map->GetMapWidth(), tokencnt*FONT_HEIGHT, "unknown token %d", o->m_data[j]);
				break;
			}
			j+=getnumparams(o->m_data[j]);
			tokencnt++;
		}
	}
	if(objsel!=-1)
	{
		SDL_Rect r={objs[objsel].m_xpos,objs[objsel].m_ypos};
		SDL_BlitSurface(marker, NULL, scr, &r);
		if(scriptsel!=-1)
		{
			r.x=map->GetMapWidth();r.y=scriptsel*FONT_HEIGHT;
			SDL_BlitSurface(marker, NULL, scr, &r);
			int* token;
			if((token=gettoken(objs[objsel].m_data, objs[objsel].m_datasize, scriptsel)))
			{
				if(*token==s_waypoint)
				{
					r.x=*(token+1);r.y=*(token+2);
					SDL_BlitSurface(marker, NULL, scr, &r);
				}
			}
		}

		// display initial coords
		drawtext(scr, map->GetMapWidth(), map->GetMapHeight(), "Pos X %d", objs[objsel].m_xpos);
		drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+FONT_HEIGHT, "Pos Y %d", objs[objsel].m_ypos);

		drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+FONT_HEIGHT*2, "Ident %s%s", objs[objsel].m_identifier,enteri?"_":"");
		drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+FONT_HEIGHT*3, "Desc %s%s", objs[objsel].m_description,enterd?"_":"");
		drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+FONT_HEIGHT*4, "SprtNam %s%s", objs[objsel].m_spritename,enters?"_":"");

		// display init params
		drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+6*FONT_HEIGHT, "[F12] visible %s", objs[objsel].INIT_GENERIC_INVISIBLE?"no":"yes");
		drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+7*FONT_HEIGHT, "[L]UA script %s", objs[objsel].INIT_GENERIC_HAS_LUASCRIPT?"on":"off");
		drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+8*FONT_HEIGHT, "[F11] animdir %s", direction_name[objs[objsel].INIT_GENERIC_ANIMDIR]);
		switch(objs[objsel].m_type)
		{
		case t_camera:
			drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+9*FONT_HEIGHT, "[F5/6] angle %d", objs[objsel].INIT_CAM_ANGLE);
			break;
		case t_sprite:
			drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+9*FONT_HEIGHT, "[F5] collision %s", objs[objsel].INIT_SPRITE_COLLISION?"on":"off");
			drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+10*FONT_HEIGHT, "[M]ap collision %s", objs[objsel].INIT_SPRITE_MAPCOLLISION?"on":"off");
			drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+11*FONT_HEIGHT, "Sho[V]able %s", objs[objsel].INIT_SPRITE_SHOVABLE?"on":"off");
			break;
		case t_door:
			drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+9*FONT_HEIGHT, "[F5/6] open pos. X %d", objs[objsel].INIT_DOOR_OPENPOSX);
			drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+10*FONT_HEIGHT, "[F7/8] open pos. Y %d", objs[objsel].INIT_DOOR_OPENPOSY);
			break;
		case t_swag:
			drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+9*FONT_HEIGHT, "[F5/6] score %d", objs[objsel].INIT_SWAG_SCORE);
			break;
		case t_finish:
			drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+9*FONT_HEIGHT, "[F5/6] end pos. X %d", objs[objsel].INIT_FINISH_ENDPOSX);
			drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+10*FONT_HEIGHT, "[F7/8] end pos. Y %d", objs[objsel].INIT_FINISH_ENDPOSY);
			break;
		default:
			break;
		}
	}
	
	// draw lines showing map limits
	drawline(scr, 0, map->GetMapHeight(), map->GetMapWidth()+RIGHT_AREA, map->GetMapHeight(),false);
	drawline(scr, map->GetMapWidth(), 0, map->GetMapWidth(), map->GetMapHeight()+BOTTOM_AREA, false);

	drawtext(scr, map->GetMapWidth(), map->GetMapHeight()+BOTTOM_AREA-FONT_HEIGHT, "Gridlock %s",locktogrid?"on":"off");

	int y=map->GetMapHeight();
	int x=0;
	
	static char* hints[]={"[ESC] Quit","[F10] Save","----------------","[UP] Prev.token","[DOWN] Next tkn.","Un[SPACE]lect","[DEL]ete",
		"[BS] Gridlock","----------------","[1/2] Tkn.par.1","[3/4] Tkn.par.2","----------------","[O] New Sprite","New [D]oor","New [P]layer","New [E]xplosive",
		"New Sc[R]iptable","New [G]uard","New [S]wag",
		"New [C]amera","New [I]tem","[F]inish Line","----------------","[W]aypoint","[Q] Sprite ref.", "[END] of data" ,0};
	int i=0;
	while(hints[i])
	{
		drawtext(scr, x, y, hints[i]);
		y+=FONT_HEIGHT;
		if(y+FONT_HEIGHT>=map->GetMapHeight()+BOTTOM_AREA)
		{
			x+=17*FONT_WIDTH;
			y=map->GetMapHeight();
		}
		i++;
	}
	SDL_UpdateRect(scr, 0, 0, scr->w, scr->h);
}


