// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <assert.h>
#include "leveled.h"

int main(int argc, char** argv)
{
	// load object graphics
	for(int i=0; i<t_finish; i++)
	{
		if(!(typegfx[i]=SDL_LoadBMP(tgfx[i])))
		{
			fprintf(stderr, "could not load %s\n", tgfx[i]);
			exit(1);
		}
		SDL_SetColorKey(typegfx[i], SDL_SRCCOLORKEY, 0);
	}

	// load marker gfx
	marker=SDL_LoadBMP("../gfx/marker.bmp");
	SDL_SetColorKey(marker, SDL_SRCCOLORKEY, 0);

	// load font
	font=SDL_LoadBMP("../gfx/console_font.bmp");
	SDL_SetColorKey(font, SDL_SRCCOLORKEY, 0);

	object objs[MAX_OBJECTS];
	int objnum=0;
	int selectedobject=-1;
	int selectedtoken=-1;

	// init SDL
	SDL_Init(SDL_INIT_VIDEO);
	SDL_EnableUNICODE(1);
	atexit(SDL_Quit);

	// load map
	SDLMappy* map=new SDLMappy;
	if(map->MapLoad(argv[1])) printf("shit\n");
	map->MapInitAnims();

	// set up screen
	SDL_Surface* screen;
	screen=SDL_SetVideoMode(map->GetMapWidth()+RIGHT_AREA, map->GetMapHeight()+BOTTOM_AREA, 16, SDL_SWSURFACE);
	map->MapMoveTo(0,0);

	// event loop
	SDL_Event evt;
	bool quit=false;
	bool doredraw=true;
	bool locktogrid=true;

	FILE* out;
	int tmp;
	char outname[strlen(argv[1])+1];
	strncpy(outname, argv[1], strlen(argv[1])-4); outname[strlen(argv[1])-4]=0;
	strcat(outname, ".lvl");

	struct stat s;
	if(!stat(outname, &s))
	{
		FILE* in=fopen(outname, "r");
		fread(&objnum, sizeof(int), 1, in);
		fread(&tmp, sizeof(int), 1, in);
		assert(tmp==sizeof(object));
		for(int i=0;i<objnum;i++)
			fread(&objs[i], sizeof(object), 1, in);
		fclose(in);
	}

	stat(argv[1], &s);
	time_t maptime=s.st_mtime;

	int step;
	bool enterident=false;
	bool enterdesc=false;
	bool entersprite=false;
	bool buttondown=false;
	while(!quit)
	{
		stat(argv[1], &s);
		if(maptime!=s.st_mtime) { map->MapLoad(argv[1]); doredraw=true; }

		if(doredraw) redraw(screen, map, objs, objnum, selectedobject, selectedtoken, locktogrid,enterident,enterdesc, entersprite);
		SDL_WaitEvent(&evt);
		doredraw=true;
		int x,y;
		SDL_GetMouseState(&x, &y);
		if(locktogrid)
		{
			x=(x&~(OBJECT_SIZE-1))+OBJECT_SIZE/2;
			y=(y&~(OBJECT_SIZE-1))+OBJECT_SIZE/2;
		}

		switch(evt.type)
		{
		case SDL_KEYDOWN:
			if(enterident||enterdesc||entersprite)
			{
				object* o=&objs[selectedobject];
				if(evt.key.keysym.sym==SDLK_RETURN)
				{
					enterident=enterdesc=entersprite=false;
				}
				else if(evt.key.keysym.sym==SDLK_BACKSPACE)
				{
					if(enterident && strlen(o->m_identifier)>0)	o->m_identifier[strlen(o->m_identifier)-1]=0;
					if(enterdesc && strlen(o->m_description)>0)	o->m_description[strlen(o->m_description)-1]=0;
					if(entersprite && strlen(o->m_spritename)>0) o->m_spritename[strlen(o->m_spritename)-1]=0;
				}
				else
				{
					char buf[2];
					buf[0]=(evt.key.keysym.unicode)&255;
					//printf("char %d\n", buf[0]);
					if(enterident) strcat(o->m_identifier,buf);
					if(enterdesc) strcat(o->m_description, buf);
					if(entersprite)	strcat(o->m_spritename, buf);
				}
			}
			else
			{
				step=evt.key.keysym.mod&KMOD_LSHIFT?10:1;
				switch(evt.key.keysym.sym)
				{
				case SDLK_INSERT:	// enter identifier
					if(selectedobject!=-1) enterident=true;
					break;
				case SDLK_HOME:
					if(selectedobject!=-1) enterdesc=true;
					break;
				case SDLK_PAGEUP:
					if(selectedobject!=-1) entersprite=true;
					break;
				case SDLK_ESCAPE:	// quit
					quit=true;
					break;
				case SDLK_SPACE:	// unselect (first token, then object)
					if(selectedtoken!=-1)
						selectedtoken=-1;
					else
						selectedobject=-1;
					break;
				case SDLK_DOWN:		// select next object/token
					if(selectedobject!=-1 && selectedtoken==-1 && selectedobject+1<objnum)
						selectedobject++;
					if(selectedobject!=-1 && selectedtoken!=-1 && gettoken(objs[selectedobject].m_data, objs[selectedobject].m_datasize, selectedtoken+1))
						selectedtoken++;
					break;
				case SDLK_UP:		// select previous object/token
					if(selectedobject!=-1 && selectedtoken==-1 && selectedobject>0)
						selectedobject--;
					if(selectedobject!=-1 && selectedtoken!=-1 && selectedtoken-1>=0)
						selectedtoken--;
					break;

				case SDLK_F10:		// save
					out=fopen(outname, "w");
					fwrite(&objnum, sizeof(int), 1, out);
					tmp=sizeof(object);
					fwrite(&tmp, sizeof(int), 1, out);
					for(int i=0;i<objnum;i++)
						fwrite(&objs[i], sizeof(object), 1, out);
					fclose(out);
					break;
				case SDLK_BACKSPACE:	// toggle gridlock
					locktogrid=!locktogrid;
					break;

					// generate new objects
#define NEWOBJECT(key, type) case key: \
	if(x<map->GetMapWidth() && y<map->GetMapHeight()) newobject(type, objs, objnum, x, y); \
	break;

					NEWOBJECT(SDLK_p, t_levelplayer);
					NEWOBJECT(SDLK_g, t_guard);
					NEWOBJECT(SDLK_c, t_camera);
					NEWOBJECT(SDLK_o, t_sprite);
					NEWOBJECT(SDLK_e, t_explosive);
					NEWOBJECT(SDLK_s, t_swag);
					NEWOBJECT(SDLK_i, t_item);
					NEWOBJECT(SDLK_d, t_door);
					NEWOBJECT(SDLK_r, t_scriptable);
					NEWOBJECT(SDLK_f, t_finish);

					// modify object position
#define SETPOS(key, pos, signum) case key: \
	if(selectedobject!=-1) \
	{ \
		objs[selectedobject].pos+=signum*step; \
	} \
	break;

					SETPOS(SDLK_F1, m_xpos, -1);
					SETPOS(SDLK_F2, m_xpos, 1);
					SETPOS(SDLK_F3, m_ypos, -1);
					SETPOS(SDLK_F4, m_ypos, 1);

					// modify init values (F keys)
				case SDLK_F6:
					if(selectedobject!=-1)
					{
						object* o=&objs[selectedobject];
						if(o->m_type==t_camera)	o->INIT_CAM_ANGLE+=step;
						else if(o->m_type==t_door) o->INIT_DOOR_OPENPOSX+=step;
						else if(o->m_type==t_swag) o->INIT_SWAG_SCORE+=step;
						else if(o->m_type==t_finish) o->INIT_FINISH_ENDPOSX+=step;
					}
					break;
				case SDLK_F5:
					if(selectedobject!=-1)
					{
						object* o=&objs[selectedobject];
						if(o->m_type==t_camera)	o->INIT_CAM_ANGLE-=step;
						else if(o->m_type==t_sprite) o->INIT_SPRITE_COLLISION=(o->INIT_SPRITE_COLLISION+1)&1;
						else if(o->m_type==t_door) o->INIT_DOOR_OPENPOSX-=step;
						else if(o->m_type==t_swag) o->INIT_SWAG_SCORE-=step;
						else if(o->m_type==t_finish) o->INIT_FINISH_ENDPOSX-=step;
					}
					break;
				case SDLK_F8:
					if(selectedobject!=-1)
					{
						object* o=&objs[selectedobject];
						if(o->m_type==t_door) o->INIT_DOOR_OPENPOSY+=step;
						else if(o->m_type==t_finish) o->INIT_FINISH_ENDPOSY+=step;
					}
					break;
				case SDLK_F7:
					if(selectedobject!=-1)
					{
						object* o=&objs[selectedobject];
						if(o->m_type==t_door) o->INIT_DOOR_OPENPOSY-=step;
						else if(o->m_type==t_finish) o->INIT_FINISH_ENDPOSY-=step;
					}
					break;
				case SDLK_F12:
					if(selectedobject!=-1)
					{
						object* o=&objs[selectedobject];
						o->INIT_GENERIC_INVISIBLE=(o->INIT_GENERIC_INVISIBLE+1)&1;
					}
					break;
				case SDLK_F11:
					if(selectedobject!=-1)
					{
						object* o=&objs[selectedobject];
						o->INIT_GENERIC_ANIMDIR=(o->INIT_GENERIC_ANIMDIR+1)&7;
					}
					break;
				case SDLK_F9:
					if(objs[selectedobject].INIT_GENERIC_HAS_LUASCRIPT && strlen(objs[selectedobject].m_identifier)>0)
					{
						char buf[80];
						strcpy(buf, "kwrite --geometry 480x480 ");
						char* buf2=strdup(objs[selectedobject].m_identifier);
						stripbrackets(buf2);
						strcat(buf, buf2);
						free(buf2);
						strcat(buf, "_script.lua &");
						buf2=escapequotes(buf);
						system(buf);
						delete[] buf2;
					}
					break;
				case SDLK_m:
					if(selectedobject!=-1)
					{
						object* o=&objs[selectedobject];
						if(o->m_type==t_sprite)
							o->INIT_SPRITE_MAPCOLLISION=(o->INIT_SPRITE_MAPCOLLISION+1)&1;
					}
					break;
				case SDLK_v:
					if(selectedobject!=-1)
					{
						object* o=&objs[selectedobject];
						if(o->m_type==t_sprite)
							o->INIT_SPRITE_SHOVABLE=(o->INIT_SPRITE_SHOVABLE+1)&1;
					}
					break;
				case SDLK_l:
					if(selectedobject!=-1)
					{
						object* o=&objs[selectedobject];
						o->INIT_GENERIC_HAS_LUASCRIPT=(o->INIT_GENERIC_HAS_LUASCRIPT+1)&1;
					}
					break;

#define ADDTOKEN(key, condition, token...) case key: \
	if(selectedobject!=-1 && (condition)) \
	{ \
			printf("adding" #token "\n"); \
			addtoscript(&objs[selectedobject], selectedtoken, token); \
			if(selectedtoken!=-1 && selectedtoken+1<objs[selectedobject].m_datasize) selectedtoken++; \
	} \
	break;

					ADDTOKEN(SDLK_w, x<map->GetMapWidth() && y<map->GetMapHeight(), s_waypoint, x-OBJECT_SIZE/2, y-OBJECT_SIZE/2);
					ADDTOKEN(SDLK_q, 1, s_sprite, 60);
					ADDTOKEN(SDLK_END, 1, s_enddata);

				case SDLK_DELETE:	// delete object/token
					if(selectedtoken!=-1)
					{
						// delete selected token
						object* o=&objs[selectedobject];
						int* tok=gettoken(o->m_data, o->m_datasize, selectedtoken);
						int size=getnumparams(*tok)+1;
						o->m_datasize-=size;

						for(int i=tok-o->m_data;i<o->m_datasize;i++)
							o->m_data[i]=o->m_data[i+size];

						// check if selectedtoken is still valid
						if(!gettoken(o->m_data,o->m_datasize,selectedtoken)) selectedtoken--;
					}
					else if(selectedobject!=-1)
					{
						// delete selected object
						objnum--;

						for(int i=selectedobject;i<objnum;i++)
							memcpy(&objs[i], &objs[i+1], sizeof(object));
						
						// remove all s_sprite references to this object
						for(int i=0;i<objnum;i++)
						{
							object* o=&objs[i];
							for(int j=0;j<o->m_datasize;j++)
							{
								switch(o->m_data[j])
								{
								case s_waypoint: j+=2; break;
								case s_sprite:
									if(o->m_data[j+1]==selectedobject) o->m_data[j+1]=-1;
									j++;
									break;
								case s_enddata: break;
								default:
									fprintf(stderr, "invalid token %d when checking sprite refs\n",o->m_data[j]);
									exit(1);
								}
							}
						}

						selectedobject=-1;
					}
					break;

					// change parameters for selected token
#define ADDVAL(kkey, param, delta) case kkey: \
	if(selectedtoken!=-1) \
	{ \
		object* o=&objs[selectedobject]; \
		int* tok=gettoken(o->m_data, o->m_datasize, selectedtoken); \
		if(getnumparams(*tok)>param && ((*tok!=s_sprite) || param>0 || (*(tok+1)+delta>=0 && *(tok+1)+delta<objnum))) \
		if(evt.key.keysym.mod&KMOD_LCTRL) \
		{ \
			if(delta>0) *(tok+param+1)<<=1;	\
			else *(tok+param+1)>>=1; \
		} else \
		(*(tok+param+1))+=delta*(evt.key.keysym.mod&KMOD_LSHIFT?10:1); \
	} \
	break;

					ADDVAL(SDLK_2, 0, 1);
					ADDVAL(SDLK_1, 0, -1);
					ADDVAL(SDLK_4, 1, 1);
					ADDVAL(SDLK_3, 1, -1);

				default:
					break;
				}
				break;
			}
			break;
		case SDL_MOUSEBUTTONDOWN:
			buttondown=true;
			if(selectedtoken!=-1 && *gettoken(objs[selectedobject].m_data, objs[selectedobject].m_datasize, selectedtoken)==s_sprite)
			{	// select a sprite reference for s_sprite
				int* token=gettoken(objs[selectedobject].m_data, objs[selectedobject].m_datasize, selectedtoken);
				for(int i=0; i<objnum; i++)
				{
					object* o=&objs[i];
					int ox=o->m_xpos;
					int oy=o->m_ypos;

					// check for an object under the mouse cursor
					if(ox<=x && oy<=y && ox+OBJECT_SIZE>=x && oy+OBJECT_SIZE>=y)
					{
						*(token+1)=i;
					}
				}
				buttondown=false;
			} else
			if(x<map->GetMapWidth())	// click on map
			{
				// nothing selected -> select an object / s_move token
				bool selectedsth=false;
				for(int i=0; i<objnum; i++)
				{
					object* o=&objs[i];
					int ox=o->m_xpos;
					int oy=o->m_ypos;

					// check for an object under the mouse cursor
					if(ox<=x && oy<=y && ox+OBJECT_SIZE>=x && oy+OBJECT_SIZE>=y)
					{
						selectedobject=i;
						selectedsth=true;
						break;
					}

					// check for a s_waypoint token under the mouse cursor
					int tokencount=0;
					int* token;
					while((token=gettoken(o->m_data, o->m_datasize, tokencount)))
					{
						if(*token==s_waypoint)
						{
							token++;
							if(*token<=x && *(token+1)<=y && *token+OBJECT_SIZE>=x && *(token+1)+OBJECT_SIZE>=y)
							{
								selectedobject=i;
								selectedtoken=tokencount;
								selectedsth=true;
								break;
							}
						}
						tokencount++;
					}
					//if(selectedtoken!=-1) break;
				}
				if(!selectedsth) selectedtoken=selectedobject=-1;
			}
			else		// click on script
			{
				x-=map->GetMapWidth();
				if(selectedobject!=-1)
				{
					// select script token
					object* o=&objs[selectedobject];
					printf("select script token at %d from %d tokens\n", y/FONT_HEIGHT,o->m_datasize);
					if(gettoken(o->m_data, o->m_datasize, y/FONT_HEIGHT))
						selectedtoken=y/FONT_HEIGHT;
					else
						selectedtoken=selectedobject=-1;
				}
				else
					selectedobject=selectedtoken=-1;
			}
			break;
		case SDL_MOUSEBUTTONUP:
			buttondown=doredraw=false;
			break;
		case SDL_MOUSEMOTION:
			if(buttondown && x<map->GetMapWidth() && y<map->GetMapHeight())
			{
				if(selectedtoken!=-1)	// change param for script token
				{
					// find token
					object* o=&objs[selectedobject];
					//printf("fetching token %d\n", selectedtoken);
					int* token=gettoken(o->m_data,o->m_datasize,selectedtoken);
					//printf("got offset %d\n", token-o->m_data);
					switch(*token)
					{
					case s_waypoint:
						*(token+1)=x-OBJECT_SIZE/2;
						*(token+2)=y-OBJECT_SIZE/2;
						break;
					default:
						break;
					}
				}
				else	// no token selected
				{
					if(selectedobject!=-1)	// object selected -> move it
					{
						objs[selectedobject].m_xpos=x-OBJECT_SIZE/2;
						objs[selectedobject].m_ypos=y-OBJECT_SIZE/2;
					}
				}
			}
			else
				doredraw=false;
			break;
		default:
			doredraw=false;
		}
	}
}
