// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include "leveled.h"
#include <assert.h>
#include <unistd.h>

int getnumparams(int token)
{
	switch(token)
	{
	case s_enddata:
		return 0;
	case s_sprite:
		return 1;
	case s_waypoint:	// two parameters
		return 2;
	}
	fprintf(stderr, "getnumparams: unknown token %d\n", token);
	return -1;
}

/// Find the nth token in the given script.
/// \param script A script.
/// \param size The size of the script.
/// \param token The number of the token to search for.
/// \returns Pointer to the token or NULL if not found.
int* gettoken(int* script, int size, int token)
{
	int tokencnt=0;
	int i=0;
	while(tokencnt!=token && i<size)
	{
		// skip the token's parameters
		i+=getnumparams(script[i]);
		tokencnt++;
		i++;
	}
	if(i>=size) return 0;
	else return &script[i];
}

void addtoscript(object* obj, int selectedtoken, script token, int par0, int par1, int par2, int par3, int par4)
{
	int insertat;
	if(selectedtoken==-1) insertat=obj->m_datasize;
	else
		insertat=gettoken(obj->m_data, obj->m_datasize, selectedtoken)-obj->m_data;
	

	printf("inserting at %d\n", insertat);
	//sleep(5);
	obj->m_datasize+=getnumparams(token)+1;
	for(int i=obj->m_datasize-1; i>=insertat+getnumparams(token)+1;i--)
	{
		obj->m_data[i]=obj->m_data[i-getnumparams(token)-1];
	}
	printf("done shifting\n");
	switch(getnumparams(token))
	{
	case 0:	// no parameters
		obj->m_data[insertat++]=token;
		break;
	case 1:	// one parameter
		obj->m_data[insertat++]=token;
		obj->m_data[insertat++]=par0;
		break;
	case 2:
		obj->m_data[insertat++]=token;
		obj->m_data[insertat++]=par0;
		obj->m_data[insertat++]=par1;
		break;
	case 3:		// three parameters
		obj->m_data[insertat++]=token;
		obj->m_data[insertat++]=par0;
		obj->m_data[insertat++]=par1;
		obj->m_data[insertat++]=par2;
		break;
	case 4:		// four parameters
		obj->m_data[insertat++]=token;
		obj->m_data[insertat++]=par0;
		obj->m_data[insertat++]=par1;
		obj->m_data[insertat++]=par2;
		obj->m_data[insertat++]=par3;
		break;
	default:
		assert(0);
	}
	if(token==s_sprite) obj->m_data[insertat-1]=-1; // undefined
	printf("script dump:");
	for(int i=0;i<obj->m_datasize;i++)
		printf(" %d", obj->m_data[i]);
	printf("\n");
}

void newobject(types t, object* objs, int& objnum, int x, int y)
{
	objs[objnum].m_xpos=x-OBJECT_SIZE/2;
	objs[objnum].m_ypos=y-OBJECT_SIZE/2;
	objs[objnum].m_type=t;
	objs[objnum].m_datasize=0;
	objs[objnum].INIT_GENERIC_INVISIBLE=0;
	objs[objnum].INIT_GENERIC_HAS_LUASCRIPT=0;
	objs[objnum].INIT_GENERIC_ANIMDIR=0;
	if(t==t_camera) objs[objnum].INIT_CAM_ANGLE=0;
	if(t==t_sprite)
	{
		objs[objnum].INIT_SPRITE_COLLISION=0;
		objs[objnum].INIT_SPRITE_MAPCOLLISION=0;
		objs[objnum].INIT_SPRITE_SHOVABLE=0;
	}
	if(t==t_door)
	{
		 objs[objnum].INIT_DOOR_OPENPOSX=objs[objnum].m_xpos;
		 objs[objnum].INIT_DOOR_OPENPOSY=objs[objnum].m_ypos;
	}
	if(t==t_swag) objs[objnum].INIT_SWAG_SCORE=0;
	objs[objnum].m_identifier[0]=0;
	if(t==t_finish)
	{
		objs[objnum].INIT_FINISH_ENDPOSX=objs[objnum].m_xpos+32;
		objs[objnum].INIT_FINISH_ENDPOSY=objs[objnum].m_ypos;
		strcpy(objs[objnum].m_identifier, "finish_line");
	}
	objs[objnum].m_description[0]=0;
	objs[objnum].m_spritename[0]=0;
	objnum++;
}

char* escapequotes(char* x)
{
	char* y=new char[strlen(x)*2+1];
	char* yy=y;
	while(*x)
	{
		if(*x=='"')	*yy++='\\';
		*yy++=*x++;
	}
	return y;
}

char* stripbrackets(char* x)
{
	char* y=new char[strlen(x)+1];
	char* yy=y;
	char* xx=x;
	while(*xx)
	{
		if(*xx!='[' && *xx!=']') *yy++=*xx;
		xx++;
	}
	*yy=0;
	strcpy(x, y);
	delete[] y;
	return x;
}
