/*
** $Id: lstate.h,v 1.41 2000/10/05 13:00:17 roberto Exp $
** Global State
** See Copyright Notice in lua.h
*/

#ifndef lstate_h
#define lstate_h

#include "lobject.h"
#include "lua.h"
#include "luadebug.h"



typedef TObject *StkId;  /* index to stack elements */


/*
** marks for Reference array
*/
#define NONEXT          -1      /* to end the free list */
#define HOLD            -2
#define COLLECTED       -3
#define LOCK            -4


struct Ref {
  TObject o;
  int st;  /* can be LOCK, HOLD, COLLECTED, or next (for free list) */
};


struct lua_longjmp;  /* defined in ldo.c */
struct TM;  /* defined in ltm.h */


typedef struct stringtable {
  int size;
  lint32 nuse;  /* number of elements */
  TString **hash;
} stringtable;



/*
** information about a call (for debugging)
** yield patch: added members to help unwind luaV_execute()'s recursion in OP_CALL/OP_TAILCALL.
*/
typedef struct CallInfo {
  struct Closure *func;  /* function being called */
  const Instruction **pc;  /* current pc of called function */
  int lastpc;  /* last pc traced */
  int line;  /* current line */
  int refi;  /* current index in `lineinfo' */
  
  /* yield patch */
  /* locals & args from luaV_execute -- must set these up before calling luaV_execute() */
  StkId base;
  const Instruction* savedpc;
  TString **const kstr;
  lua_Hook linehook;
  int mustreturn; /* flag that tells luaV_execute() when it must return */

  /* locals & args from luaD_call */
  int nResults;  /* passed from luaD_prolog() to luaD_epilog */
  StkId funcid;  /* passed from luaD_prolog() to luaD_epilog */
  lua_Hook callhook;  /* passed from luaD_prolog() to luaD_epilog */
  StkId firstResult;  /* return value from luaV_execute to luaD_epilog */

  /* link, to form a stack. */
  struct CallInfo* prev;

} CallInfo;


struct lua_State {
  /* thread-specific state */
  StkId top;  /* first free slot in the stack */
  StkId stack;  /* stack base */
  StkId stack_last;  /* last free slot in the stack */
  int stacksize;
  StkId Cbase;  /* base for current C function */
  struct lua_longjmp *errorJmp;  /* current error recover point */
  char *Mbuffer;  /* global buffer */
  size_t Mbuffsize;  /* size of Mbuffer */
  CallInfo* callinfostack;
  int callcount;	/* yield patch: recursion count for luaD_call().  Use this to prevent trying to unwind past more than one luaD_call() */
  StkId yield_oldCbase;  /* yield patch: stash for luaD_runprotected. */
  StkId yield_oldtop;  /* yield patch: stash for luaD_runprotected. */
  int yield_allowhooks;  /* yield patch: stash for luaD_runprotected. */
  /* global state */
  Proto *rootproto;  /* list of all prototypes */
  Closure *rootcl;  /* list of all closures */
  Hash *roottable;  /* list of all tables */
  stringtable strt;  /* hash table for strings */
  stringtable udt;   /* hash table for udata */
  Hash *gt;  /* table for globals */
  struct TM *TMtable;  /* table for tag methods */
  int last_tag;  /* last used tag in TMtable */
  struct Ref *refArray;  /* locked objects */
  int refSize;  /* size of refArray */
  int refFree;  /* list of free positions in refArray */
  unsigned long GCthreshold;
  unsigned long nblocks;  /* number of `bytes' currently allocated */
  lua_Hook callhook;
  lua_Hook linehook;
  int allowhooks;
  CallInfo* callinfofreelist;	/* yield patch: stash unused callinfo structs here, so we don't have to constantly alloc/dealloc. */
};


#endif

