-- test the yield patch.  The yield patch adds a yield() function,
-- which suspends script execution in a restartable state (via the API
-- function lua_resume(L)).  The yield patch also implements true tail
-- recursion for the OP_TAILCALL opcode.


function donothing_tailrec(x)
	return x
end

function donothing_rec1(x)
	return donothing_tailrec(x)
end

function donothing_rec2(x)
	for i = 1,1000 do
		donothing_rec1(x)
	end
	return donothing_rec1(x)
end

function donothing_rec3(x)
	for i = 1,1000 do
		donothing_rec2(x)

		-- exit script execution, returning LUA_YIELD to the caller.
		-- The caller can resume the script via lua_resume(L).
		yield()
	end
	return donothing_rec2(x)
end

c = clock()
r = donothing_rec3(10)
t = clock() - c
print("time = " .. t .. ", result = " .. r);


count = 0

function increment_tailrec(ct)
-- stupid recursive function to increment 'count'
	if ct <= 0 then
		return 0
	else
		count = count + 1
		return increment_tailrec(ct-1)	-- for some reason, we must return a value for OP_TAILCALL to be generated.
	end
end


function test(x)
	count = 0
	increment_tailrec(x)
	if count == x then
		print("success --> count = " .. x);
	else
		print("failure --> count = " .. count .. " but x = " .. x);
	end
end


-- The yield patch also implements proper tail recursion for the
-- OP_TAILCALL opcode.  In unpatched Lua 4.0, one of the following
-- calls will probably cause a stack overflow error.
test(1)
test(10)
test(100)
test(1000)
test(10000)
test(100000)
test(1000000)
