// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include "mansion.h"
#include "messagebox.h"
#include "vbl.h"
#include "input.h"

#include "mansion_data.c"

extern const palettedata sprite_palette;
#include "mansion_lvl_gfx.h"

extern const spritedata frame_sprite;
extern const fontdata console_font;
extern const palettedata msgbox_palette;
extern const spritedata portrait_sprite;

mansion::mansion(int first_bg)
: level(first_bg, &mansion_map)
{
	setdefaultcollidemap(1);

	setexcess(0,0,12,0);
	setscrollbordery(60);
	setscrollborderx(110);
	setspritepalette(&sprite_palette);

#include "mansion_lvl_ctor.h"

	faris->setbounding(sprite::b_left,1);	// left edge of bounding box
	faris->setbounding(sprite::b_right,1);	// right edge of bounding box
	faris->setanimdirection(sprite::d_up);

	for(int i=0;i<3;i++)
	{
		guards[i]->setbounding(sprite::b_left,1);
		guards[i]->setbounding(sprite::b_right,1);
	}

	addtoinventory(bomb, faris);

	pics[1]->setbounding(sprite::b_left, 4);
	pics[1]->setbounding(sprite::b_right, 4);

	chest->enableautopickup(false);

	setradar(0,SCREEN_WIDTH-64, SCREEN_HEIGHT-40, 64, 40);
	
	setpos(13*16-SCREEN_WIDTH/2, 31*16-SCREEN_HEIGHT/2);
	scrolllock(faris);			// lock map to faris
	
	widget::setcharacters(&console_font);
	widget::setpalette(&msgbox_palette);

	enableradar(true);

	m_safeblown=m_secretrevealed=false;
}

mansion::~mansion()
{
}

void mansion::compute(bool lowtime)
{
	if(getscore()>0) faris->allowcrossfinishline(true);
	level::compute(lowtime);
	if(bomb->exploded())
	{
		for(int i=0; i<3; i++) guards[i]->attract(bomb->getxpos(), bomb->getypos(), 1, 1, 240);
	}
	if(!m_safeblown && !safe->isvisible() && bomb->explosionfinished())
	{
		cash->show();
		m_safeblown=true;
	}
	if(!m_secretrevealed && secret->isopen())
	{
		setexcess(0, 0, 0, 0);
		chest->show();
		key->show();
		m_secretrevealed=true;
		modifysecrets(1);
	}
}

level::state mansion::endofstage()
{
	if(!faris->isvisible())	// Blew yourself up...
		return c_lost;
	
	if(faris->hascrossedfinishline())	// reached the exit
		return c_won;

	return level::endofstage();
}

bool mansion::playerusesitem(levelplayer* who, item* what, sprite* onwhat)
{
	uprintf("%x uses %x on %x\n", (int)who, (int)what, (int)onwhat);
	
	if(what==bomb)
	{
		bomb->explode(who, onwhat, 360);
		return true;
	}
	if(what==key && onwhat==safe)
	{
		m_safeblown=true;
		cash->show();
		safe->hide();
		return true;
	}
	return false;
}
