// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <menu.h>
#include <misc.h>
#include <input.h>

menu::menu(int bg, const spritedata* border, const backdropdata* backdrop, const char* headline, const char** items, int numitems)
: widget(bg, border, 0, 22, 16, 32, 16, 0, true)
{
	int maxlen=0;
	int tmp;

	drawstring(11-strlen(headline)/2, 1, headline);

	for (int i=0; i<numitems; i++)
	{
		drawstring(4, 4+i*2, items[i]);
		tmp=strlen(items[i]);
		if (tmp>maxlen)	maxlen=tmp;
	}

	ham_SetBgMode(0);

	setbackgroundimage(backdrop);

	bg+=2;	// skip bgimage bg

	ham_bg[bg].ti=ham_bg[m_bg].ti; // reusing the widget frame's tiles. Watch out when deleting!
	ham_bg[bg].mi=ham_InitMapEmptySet(0,0);

	// init map to transparent tile; ham_InitMapEmptySet() should do this, but it fails occasionally
	for (int i=0;i<32;i++)
		for (int j=0;j<32;j++)
			spam_SetMapTileEx(bg, i, j, 0, 0, 0, 15);


	// draw border

	// corners
	// ham_SetMapTileEx() fails in 16-color modes with cbb_only_mode turned off. Using my own implementation instead.
	spam_SetMapTileEx(bg, 0, 0, t_tl,0,0,15);
	spam_SetMapTileEx(bg, maxlen+1, 0, t_tr,0,0,15);
	spam_SetMapTileEx(bg, 0, 3, t_bl,0,0,15);
	spam_SetMapTileEx(bg, maxlen+1, 3, t_br,0,0,15);

	// edges
	for (int i=1;i<maxlen+1;i++)
	{
		spam_SetMapTileEx(bg, i, 0, t_top,0,0,15);
		spam_SetMapTileEx(bg, i, 3, t_bottom,0,0,15);
		// CONSIDERME: Why is this commented out? Could it still be needed?
		//for (int j=1;j<ysize-1;j++)		 // initialise widget body
		//	spam_SetMapTileEx(bg, i, j, t_empty,0,0,15);
	}
	for (int i=1;i<3;i++)
	{
		spam_SetMapTileEx(bg, maxlen+1, i, t_right,0,0,15);
		spam_SetMapTileEx(bg, 0, i, t_left,0,0,15);
	}
	
	ham_InitBg(bg, 1, 0, 0);
	M_BGXCNT_PRIO_SET(m_bg, 1);	// make sure our marker is in front of the main widget
}

menu::~menu()
{
	ham_DeInitMapSet(ham_bg[m_bg+2].mi);
	M_DISCNT_BGX_SET(m_bg+2,0);
}

int menu::getselection(int bg, const spritedata* border, const backdropdata* backdrop, const char* headline, const char** items, int numitems)
{
	menu* m=new menu(bg, border, backdrop, headline, items, numitems);
	int selection=0;

	while (!input::action())
	{
		ham_SetBgXY(m->m_bg+2, (u16)-(32+3*8), (u16)-(0+(5+selection*2)*8));
		if (input::down() && selection+1<numitems)
		{
			selection++;
			while (input::down());
		}
		if (input::up() && selection>0)
		{
			selection--;
			while (input::up());
		}
	}
	delete m;
	return selection;
}
