// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <mygba.h>
#include <messagebox.h>

#include <misc.h>
#include <vbl.h>
#include <input.h>

messagebox::messagebox(bool modal, int bg, const char* text, const spritedata* border, const portraitdata* portrait,
					   int xsize, int ysize, int xpos, int ypos, int textdelay)
: widget(bg, border, portrait?portrait->xsize*portrait->ysize:0, xsize, ysize, xpos, ypos, textdelay)
{
	int i;

	m_text=text;
	m_position=0;
	m_writedone=false;

	m_portrait=portrait;
	m_pxsize=portrait->xsize;
	m_pysize=portrait->ysize;

	if (m_portrait)
	{
		ASSERT(xsize-2 >= m_pxsize && ysize-2 >= m_pysize, "msgbox too small for portrait");

		ham_ReloadTileGfx(ham_bg[bg].ti, (u16*)(m_portrait->graphics), widget::t_startofaddongfx, m_pxsize*m_pysize);

		ham_LoadBGPal16((void*)(portrait->palette), 14);

		// draw portrait
		int tmpy=0;
		int tmpx=0;
		for (i=0;i<m_pxsize*m_pysize;i++)
		{
			spam_SetMapTileEx(bg, tmpx+1, tmpy+1, widget::t_startofaddongfx+i, 0, 0, 14);
			if (++tmpx == m_pxsize)
			{
				tmpx=0;
				tmpy++;
			}
		}
	}

	if(!modal) return;

	while(input::action());
	while(writenextpage())
	{
		while(input::action());
		while(!input::action());
		while(input::action());
	}

	delete this;
}

//#define DEBUG_WRITENEXTPAGE

bool messagebox::writenextpage()
{
	int i;		// line iterator
	int width;	// width of writable area
	int start;	// beginning of writable area

	if(m_writedone) return false;

	for(i=m_borderwidth;i<m_ysize-m_borderwidth*2;i+=W_CHAR_HEIGHT)
	{
		width=m_xsize-m_borderwidth*2;
		start=m_borderwidth;

		// don't overwrite the portrait if present
		if(m_portrait && i<=m_pysize)
		{
			width-=m_pxsize;
			start+=m_pxsize;
		}

		// clear this line
#ifdef DEBUG_WRITENEXTPAGE
		uprintf("start %d width %d m_xsize %d m_pxsize %d\n",start,width,m_xsize,m_pxsize);
#endif
		for(int j=start;j<start+width;j++)
		{
			spam_SetMapTileEx(m_bg, j, i, t_empty, 0, 0, 15);
			spam_SetMapTileEx(m_bg, j, i+1, t_empty, 0, 0, 15);
		}
	}

	for(i=m_borderwidth;i<m_ysize-m_borderwidth*2;i+=W_CHAR_HEIGHT)
	{
		const char* tstart=m_text+m_position;		// where we left off last time
		width=m_xsize-m_borderwidth*2;
		start=m_borderwidth;


		// don't overwrite the portrait if present
		if(m_portrait && i<=m_pysize)
		{
			width-=m_pxsize;
			start+=m_pxsize;
		}

		if(m_writedone) continue;

		// complete string fits this line
		if((int)strlen(tstart)<=width)
		{
			drawstring(start, i, tstart);
			m_writedone=true;
			continue;	// cannot return here, lines below may need to be cleared
		}

		// start at end of line, then trace back to the last SPACE char
		const char* t=tstart+width;
		while(t[0]!=' ' && t>tstart) t--;
		
		// no SPACE found
		if(t==tstart)
		{
			drawstringn(start, i, t, width);
			m_position+=width;
			continue;
		}

		drawstringn(start, i, tstart, t-tstart);
		m_position+=t-tstart+1;	// The '1' skips the SPACE char.
	}
	return true;
}
