// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

extern "C" {
#include <mygba.h>
}

#include <misc.h>

#if defined(__arm__) && defined(__thumb__)
#error Both __arm__ and __thumb__ defined, cannot decide which SWIs to use
#endif

/// Incredibly ugly hack. This is only there because libstdc++ calls fwrite when you try to use a pure virtual method.
#include <stdio.h>

size_t fwrite(const _PTR p, size_t _size, size_t n, FILE* fp)
{
	ASSERT(0, "fwrite called");
	return 0;
}

#include <stdlib.h>
/// The standard newlib abort() calls some random SWI, thus we have our own implementation.
void abort(void)
{
	register unsigned int lr asm("lr");
	uprintf("aborted at %x\n",lr);
	while(1);
}

/// new operator exception handler.
void my_new_handler()
{
	uprintf("new exception thrown.\n");
	printmeminfo();
	while(1);
}

#include <malloc.h>
extern struct mallinfo __malloc_current_mallinfo;
extern "C" void __malloc_update_mallinfo(void);

/// Prints info on dynamically allocated memory.
void printmeminfo(void)
{
	__malloc_update_mallinfo();
	uprintf("malloc info:\n");
	uprintf("------------\n");
	uprintf("total space allocated from system: %d\n",__malloc_current_mallinfo.arena);
	uprintf("number of non-inuse chunks: %d\n",__malloc_current_mallinfo.ordblks);
	uprintf("number of mmapped regions: %d\n",__malloc_current_mallinfo.hblks);
	uprintf("total space in mmapped regions: %d\n",__malloc_current_mallinfo.hblkhd);
	uprintf("total allocated space: %d\n",__malloc_current_mallinfo.uordblks);
	uprintf("total non-inuse space: %d\n",__malloc_current_mallinfo.fordblks);
	uprintf("top-most, releasable (via malloc_trim) space: %d\n",__malloc_current_mallinfo.keepcost);
}

extern "C" const unsigned short transparent_tile[64]={0};

/// Replacement for ham_SetMapTileEx() which does not work with 16-color BGs. Works only for non-rotscale and single-mapsection
/// BGs, e.g. radar, message box.
void spam_SetMapTileEx(u8 bgno,  u32 x, u32 y, u32 tileno, u32 hflip, u32 vflip, u32 palno)
{
	u16* dest=MEM_HAMBG_SLOT_PTR(ham_bg[bgno].mi->first_block)+x+y*32;
	*dest=tileno+
		  (32>>ham_bg[bgno].ti->col_mode)*ham_bg[bgno].ti->offset_from_cbb+	  // offset from cbb in tiles
		  (palno<<12) + (hflip<<10) + (vflip<<11);
}

/// Replacement for ham_CopyObjToOAM() which is buggy when you use ham_SetObjBefore().
void spam_CopyObjToOAM()
{
	int o=ham_obj_first;	// start of work RAM sprite list
	int i=0;				// OAM sprite slot
	do
	{
		if (!(ham_obj[o].taken && ham_obj[o].active)) {
			o=ham_obj[o].next_obj; continue;
		}
		ACCESS_16(MEM_OAM_ENTRY(i))=ham_obj[o].objatr0;
		ACCESS_16(MEM_OAM_ENTRY(i)+2)=ham_obj[o].objatr1;
		ACCESS_16(MEM_OAM_ENTRY(i)+4)=ham_obj[o].objatr2;
		o=ham_obj[o].next_obj; i++;
	} while (o!=255);

	// clear the remaining OAM slots
	for (;i<128;i++)
	{
		ACCESS_16(MEM_OAM_ENTRY(i))=ACCESS_16(MEM_OAM_ENTRY(i)+2)=241;
		ACCESS_16(MEM_OAM_ENTRY(i)+4)=0;
	}
}

/// Replacement for ham_CreateWin(). Bit 4 of fx_on enables SFX for inside, bit 0 for outside.
void spam_CreateWin(u8 winno, u8 x1, u8 y1, u8 x2, u8 y2, u8 inside_on_for, u8 outside_on_for, u8 fx_on)
{
	M_WINXX_SET(winno, x1, x2);
	M_WINXY_SET(winno, y1, y2);
	if (winno<2)
	{
		R_WININ&=0xff00>>(winno*8);
		R_WININ|=(((fx_on&2)<<1)+inside_on_for)<<(winno*8);
	} else
	{
		R_WINOUT&=0x00ff;
		R_WINOUT|=(((fx_on&2)<<1)+inside_on_for)<<8;
	}
	R_WINOUT&=0xff00;
	R_WINOUT|=outside_on_for+((fx_on&1)<<5);

	R_DISCNT&=0x1fff;
	R_DISCNT|=1<<(13+winno);
}

/// Replacement for ham_SetObjMode() which does not work at all.
void spam_SetObjMode(u8 objno, u8 obj_mode)
{
	ham_obj[objno].objatr0&=0xf3ff;
	ham_obj[objno].objatr0|=obj_mode<<10;
}

int bios_div(int num, int denom, int *rem, int* absquot)
{
	register int quot asm("r0");
	asm volatile
	(
#ifdef __arm__
	"swi   0x60000\n"
#else
	"swi 6\n"
#endif

	"ldr   r2,%0\n"
	"str   r1,[r2]\n"
	"ldr   r2,%1\n"
	"str   r3,[r2]\n"
	: "=m" (rem), "=m" (absquot) // Outputs
	:
	: "r1","r2","r3"			  // Regs crushed & smushed
	);
	return quot;
}

/// Returns the angle of the given line. Range is -180 to 180 (-0x8000 to 0x7fff).
short bios_atan2(short x, short y)
{
	register short theta asm("r0");
	asm volatile
	(
#ifdef __arm__
	"swi 0xa0000\n"
#else
	"swi 0xa\n"
#endif
	);
	return theta;
}

/// Returns the square root of the given number.
u16 bios_sqrt(u32 num)
{
	register u16 sqrt asm("r0");
	asm volatile
	(
#ifdef __arm__
	"swi 0x80000\n"
#else
	"swi 0x8\n"
#endif
	);
	return sqrt;
}

// FIXME: Does not work when inlined.
void bios_lz77uncompvram(u16* src, u16* dest)
{
#ifdef __arm__
	asm("swi 0x120000\n");
#else
	asm("swi 0x12\n");
#endif
}

// FIXME: Does not work when inlined.
void bios_lz77uncompwram(u16* src, u16* dest)
{
#ifdef __arm__
	asm("swi 0x110000\n");
#else
	asm("swi 0x11\n");
#endif
}

volatile u16 swicall[2] MEM_IN_EWRAM;
int bios_generic_junk MEM_IN_EWRAM;

int bios_generic(u8 swi, int r0, int r1, int r2, int r3, int* rr1, int* rr2, int* rr3)
{
	register int retval asm("r0");

	// neither ARM nor Thumb support SWI Rx, so we have to use self-modifying code.
#ifdef __arm__
#error not implemented yet
#endif
	swicall[0]=0xdf00+swi;	// swi
	swicall[1]=0x46a7;	// mov pc, r4
	
	asm volatile
	(
	"mov r0, %4\n"
	"mov r1, %5\n"
	"mov r2, %6\n"
	"ldr r3, %7\n"
	"mov r4, pc\n"
	"mov pc,%3\n"	// branch to swicall
	"ldr   r4,%0\n"
	"str   r1,[r4]\n"
	"ldr   r4,%1\n"
	"str   r2,[r4]\n"
	"ldr   r4,%2\n"
	"str   r3,[r4]\n"
	: "=m" (rr1), "=m" (rr2), "=m" (rr3) // Outputs
	: "r" (swicall), "r" (r0), "r" (r1), "r" (r2), "m" (r3)
	: "r1","r2","r3", "r4"			  // Regs crushed & smushed
	);
	return retval;
}
