// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <object.h>
#include <vbl.h>

//#define DEBUG_MOVETO

void object::moveto(int x, int y, int speed_num, int speed_denom)
{
	m_movecountdown=speed_denom;
	m_movespeed_num=speed_num;
	m_movespeed_denom=speed_denom;
	m_movedeltax = abs(m_xpos - x);		 // The difference between the x's
	m_movedeltay = abs(m_ypos - y);		 // The difference between the y's

	if (x >= m_xpos)				 // The x-values are increasing
	{
		m_xinc1 = 1;
		m_xinc2 = 1;
	} else
	{						   // The x-values are decreasing
		m_xinc1 = -1;
		m_xinc2 = -1;
	}

	if (y >= m_ypos)				 // The y-values are increasing
	{
		m_yinc1 = 1;
		m_yinc2 = 1;
	} else
	{						   // The y-values are decreasing
		m_yinc1 = -1;
		m_yinc2 = -1;
	}

	if (m_movedeltax >= m_movedeltay)		  // There is at least one x-value for every y-value
	{
		m_xinc1 = 0;					// Don't change the x when numerator >= denominator
		m_yinc2 = 0;					// Don't change the y for every iteration
		m_den = m_movedeltax;
		m_num = m_movedeltax / 2;
		m_numadd = m_movedeltay;
		m_numpixels = m_movedeltax;			// There are more x-values than y-values
	} else
	{						   // There is at least one y-value for every x-value
		m_xinc2 = 0;					// Don't change the x for every iteration
		m_yinc1 = 0;					// Don't change the y when numerator >= denominator
		m_den = m_movedeltay;
		m_num = m_movedeltay / 2;
		m_numadd = m_movedeltax;
		m_numpixels = m_movedeltay;			// There are more y-values than x-values
	}

	m_ismoving=true;
	m_curpixel=0;
	m_moveblocked=false;
#ifdef DEBUG_MOVETO
	uprintf("tox %d toy %d spd %d dx %d dy %d\n",x,y,m_movespeed_num,m_movedeltax,m_movedeltay);
#endif
}

void object::compute(bool lowtime)
{
	if(m_ismoving) moveme();
}

//#define DEBUG_MOVEME

void object::moveme()
{
	int newx,newy,oldx,oldy;
	if (--m_movecountdown) return;
	
	// fix-up speeds
	int sn=m_movespeed_num;
	int sd=m_movespeed_denom;
	if(VBL_FRAMESKIP>1)
	{
		if(sd>1) sd/=VBL_FRAMESKIP;		// see, this is why I told you to use powers of two for VBL_FRAMESKIP...
		else sn*=VBL_FRAMESKIP;
	}

	m_movecountdown=sd;
	newx=oldx=m_xpos;
	newy=oldy=m_ypos;
	
	
	int tmp=m_curpixel;
	for (; m_curpixel < tmp+sn; m_curpixel++)
	{
		m_num += m_numadd;				// Increase the numerator by the top of the fraction
		if (m_num >= m_den)				// Check if numerator >= denominator
		{
			m_num -= m_den;				  // Calculate the new numerator value
			newx += m_xinc1;				  // Change the x as appropriate
			newy += m_yinc1;				  // Change the y as appropriate
		}
		newx += m_xinc2;					// Change the x as appropriate
		newy += m_yinc2;					// Change the y as appropriate
	}
	
	ASSERT(newx!=oldx || newy!=oldy, "whoops...");
	setpos(newx, newy);
	if((oldx==newx || m_xpos != newx) && (oldy==newy || m_ypos != newy))
	{
#ifdef DEBUG_MOVEME
		uprintf("reqd %d/%d got %d/%d\n", newx, newy, m_xpos, m_ypos);
#endif
		m_ismoving=false;
		m_moveblocked=true;
	}

	if(m_curpixel >= m_numpixels) {
		m_ismoving=false;
	}

}

