// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <scriptable.h>
#include <stage.h>
#include <hardware.h>
extern "C" {
#include <lualib.h>
}

void scriptable::startscript(const char* script, int size, const int* data)
{
	ASSERT(!m_lua, "stop running script first");
	uprintf("starting script at %x, size %d\n", script, size);
	m_scriptsuspended=false;
	m_script=script;
	m_scriptsize=size;
	m_data=data;
	m_firstrun=true;
	m_triggersignal=m_waitforsignal=false;
	m_lua=lua_open(50);
	
	// load standard stuff, including yield()
	lua_baselibopen(m_lua);
	lua_register(m_lua, "random", lua_random);	// We don't need the entire math library.

#ifdef DEBUG_LUA
	lua_register(m_lua,"print",lua_print);
	lua_register(m_lua, LUA_ERRORMESSAGE, lua_errormessage);
#endif

	lua_pushnumber(m_lua, (int)this);
	lua_pushcclosure(m_lua, lua_this, 1); lua_setglobal(m_lua, "this");

	lua_register(m_lua, "setpos", lua_setpos);
	lua_register(m_lua, "moveto", lua_moveto);
	lua_register(m_lua, "turnto", lua_turnto);
	lua_register(m_lua, "ismoving", lua_ismoving);
	lua_register(m_lua, "setanimdir", lua_setanimdir);
	lua_register(m_lua, "wait", lua_wait);
	lua_pushnumber(m_lua, (int)this);
	lua_pushcclosure(m_lua, lua_waitmotion,1); lua_setglobal(m_lua, "waitmotion");
	lua_register(m_lua, "show", lua_show);
	lua_register(m_lua, "hide", lua_hide);
	lua_pushnumber(m_lua, (int)this);
	lua_pushcclosure(m_lua, lua_onsignal, 1); lua_setglobal(m_lua, "onsignal");
	lua_pushnumber(m_lua, (int)this);
	lua_pushcclosure(m_lua, lua_raisesig, 1); lua_setglobal(m_lua, "raisesig");
	lua_register(m_lua, "waitforsig", lua_waitforsig);

	if(data)	// These functions need a data token array, so we only register them if we have one.
	{
		lua_register(m_lua, "movetowp", lua_movetowp);
		lua_pushnumber(m_lua, (int)this);
		lua_pushcclosure(m_lua,lua_getsprite,1); lua_setglobal(m_lua, "getsprite");
	}
}

void scriptable::stopscript()
{
	ASSERT(m_lua, "no script started");
	lua_close(m_lua);
	m_lua=0;
}

scriptable::scriptable(stage* parent, const spritedata* gfx, int xpos=0, int ypos=0)
: sprite(parent, gfx, xpos, ypos)
{
	m_scriptsuspended=false;
	m_firstrun=true;
	m_lua=0;
	m_waitend=0;
	m_waiting=false;
	m_waitmotion=0;

	setdescription("An object.");
}

scriptable::~scriptable()
{
	if(m_lua) lua_close(m_lua);
}

//#define PROFILE_LUA

#ifdef PROFILE_LUA
TOOL_PROFILER_INIT;
#endif

void scriptable::compute(bool lowtime)
{
	sprite::compute(lowtime);

	if(lowtime || m_scriptsuspended)	return;

	if(m_waitmotion)
	{
		if(m_waitmotion->ismoving() || m_waitmotion->isturning())
			return;
		else
			m_waitmotion=0;
	}
	if(!m_lua || (m_waiting && m_parent->getticks()<m_waitend))	return;

	if(m_waitforsignal)
	{
		if(m_parent->israised(m_sigwaitingmask))
			m_waitforsignal=false;
		else
			return;
	}

	if(m_triggersignal && m_parent->israised(m_triggermask))
	{
		lua_getglobal(m_lua, "sighandler");
		lua_pushnumber(m_lua, m_parent->getallsignals());
		lua_call(m_lua, 1, 0);
		return;
	}

	int ret;
	if(m_firstrun)
	{
		hw::stopproftimer();	// This is a one-shot, we don't want it to skew the profile
		ret=lua_dobuffer(m_lua,m_script,m_scriptsize,0);
		hw::startproftimer();

		// just to test lua_call interrupts
		// resuming after this needs a patch on top of lua-4.0.1 + yield.patch that disables the cleaning of the callinfostack when doing a lua_call()
		//lua_getglobal(m_lua,"print");
		//lua_pushstring(m_lua,getdescription());
		//lua_call(m_lua,1,0);
		m_firstrun=false;
	}
	else
	{
#ifdef PROFILE_LUA
		TOOL_PROFILER_START;
#endif
		ret=lua_resume(m_lua);
#ifdef PROFILE_LUA
		TOOL_PROFILER_END;
#endif
	}

	if(ret != LUA_YIELD)
	{
		uprintf("script ended with %d\n",ret);
		lua_close(m_lua);
		m_lua=0;
	}

}

