// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <scriptable.h>
#include <stage.h>

extern "C" {
#include <lualib.h>
}

//#define DEBUG_VERBOSE uprintf("%s: %d\n", __FUNCTION__, __LINE__);
#define DEBUG_VERBOSE

int scriptable::lua_errormessage (lua_State *L) {
	DEBUG_VERBOSE
#ifdef DEBUG_LUA
	const char *s = lua_tostring(L, 1);
	if(s == NULL) s = "(no message)";
	uprintf("error: %s\n", s);
#endif
	return 0;
}

/* a simple "print". based on the code in lbaselib.c */
int scriptable::lua_print(lua_State *L)
{
	DEBUG_VERBOSE
#ifdef DEBUG_LUA
	int n=lua_gettop(L);
	int i;
	for(i=1; i<=n; i++)
	{
		if(i>1)	uprintf("\t");
		if(lua_isstring(L,i))
			uprintf("%s",lua_tostring(L,i));
		else
			uprintf("%s:%p",lua_typename(L,lua_type(L,i)),lua_topointer(L,i));
	}
	uprintf("\n");
#endif	
	return 0;
}

int scriptable::lua_this(lua_State* L)
{
	DEBUG_VERBOSE
	int n=lua_gettop(L);
	ASSERT(n==1, "lua_this called with parms");
	int t=lua_tonumber(L, 1);
	//uprintf("lua_this called, t=%x\n",t);
	lua_pushnumber(L, t);
	return 1;
}

int scriptable::lua_setpos(lua_State *L)
{
	DEBUG_VERBOSE
	int n=lua_gettop(L);
	ASSERT(n==3, "lua_setpos bad params");
	sprite* s=(sprite*)lua_tonumber(L, 1);
	int x=lua_tonumber(L, 2);
	int y=lua_tonumber(L, 3);
	//uprintf("setpos %s %d %d\n", s->getdescription(), x, y);
	s->setpos(x, y);
	return 0;
}

int scriptable::lua_turnto(lua_State* L)
{
	DEBUG_VERBOSE
	int n=lua_gettop(L);
	ASSERT(n==4, "lua_moveto bad params");
	sprite* s=(sprite*)lua_tonumber(L, 1);
	ASSERT(s->isrotatable(),"sprite is not rotatable");
	s->turnto(lua_tonumber(L, 2), lua_tonumber(L, 3), lua_tonumber(L, 4));
	return 0;
}

int scriptable::lua_moveto(lua_State *L)
{
	DEBUG_VERBOSE
	int n=lua_gettop(L);
	ASSERT(n==5, "lua_moveto bad params");
	sprite* s=(sprite*)lua_tonumber(L, 1);
	int x=lua_tonumber(L, 2);
	int y=lua_tonumber(L, 3);
	int sn=lua_tonumber(L, 4);
	int sd=lua_tonumber(L, 5);
	s->moveto(x, y, sn, sd);
	return 0;
}

int scriptable::lua_movetowp(lua_State* L)
{
	DEBUG_VERBOSE
	int n=lua_gettop(L);
	ASSERT(n==4, "lua_movetowp bad params");
	scriptable* s=(scriptable*)lua_tonumber(L, 1);

	int wp=lua_tonumber(L, 2);
	int c=0;
	int x=-1;
	int y=-1;
	for(int i=0; s->m_data[i]!=s_enddata; i++)
	{
		switch(s->m_data[i])
		{
		case s_waypoint:
			if(c==wp)
			{
				x=s->m_data[i+1];
				y=s->m_data[i+2];
			}
			i+=2; c++;
			break;
		case s_sprite: i++; break;
		default:
			ASSERT(0, "unexpected token");
		}
	}
	ASSERT(x!=-1 && y!=-1, "could not find waypoint");

	int sn=lua_tonumber(L, 3);
	int sd=lua_tonumber(L, 4);
	s->moveto(x, y, sn, sd);
	return 0;
}

int scriptable::lua_getsprite(lua_State* L)
{
	DEBUG_VERBOSE
	int n=lua_gettop(L);
	ASSERT(n==2, "lua_getsprite bad params");
	scriptable* s=(scriptable*)lua_tonumber(L, 2);
	int sn=lua_tonumber(L, 1);
	sprite* t=0;
	int c=0;
	for(int i=0; s->m_data[i]!=s_enddata; i++)
	{
		switch(s->m_data[i])
		{
		case s_waypoint:
			i+=2;
			break;
		case s_sprite:
			if(c==sn) t=s->m_parent->getstaticsprite(s->m_data[i+1]);
			c++; i++; break;
		default:
			ASSERT(0, "unexpected token");
		}
	}
	ASSERT(t, "unable to find sprite reference");
	lua_pushnumber(L, (int)t);
	return 1;
}

int scriptable::lua_ismoving(lua_State* L)
{
	DEBUG_VERBOSE
	int n=lua_gettop(L);
	ASSERT(n==1, "lua_ismoving bad params");
	sprite* s=(sprite*)lua_tonumber(L, 1);
	lua_pushnumber(L, s->ismoving()||s->isturning());
	return 1;
}

int scriptable::lua_waitmotion(lua_State* L)
{
	DEBUG_VERBOSE
	int n=lua_gettop(L);
	ASSERT(n==2, "lua_waitmotion bad params");
	((scriptable*)lua_tonumber(L, 2))->m_waitmotion=(sprite*)lua_tonumber(L, 1);
	lua_yield(L);
	return 0;
}

int scriptable::lua_setanimdir(lua_State* L)
{
	DEBUG_VERBOSE
	int n=lua_gettop(L);
	ASSERT(n==2, "lua_setanimdir bad params");
	sprite* s=(sprite*)lua_tonumber(L, 1);
	s->setanimdirection(lua_tonumber(L, 2));
	return 0;
}

int scriptable::lua_wait(lua_State* L)
{
	DEBUG_VERBOSE
	int n=lua_gettop(L);
	ASSERT(n==2, "lua_wait bad params");
	scriptable* t=(scriptable*)lua_tonumber(L, 1);
	t->m_waiting=true;
	t->m_waitend=t->m_parent->getticks()+lua_tonumber(L, 2);
	lua_yield(L);
	return 0;
}

int scriptable::lua_show(lua_State* L)
{
	DEBUG_VERBOSE
	int n=lua_gettop(L);
	ASSERT(n==1, "lua_show bad params");
	sprite* s=(sprite*)lua_tonumber(L, 1);
	s->show();
	return 0;
}

int scriptable::lua_hide(lua_State* L)
{
	DEBUG_VERBOSE
	int n=lua_gettop(L);
	ASSERT(n==1, "lua_hide bad params");
	sprite* s=(sprite*)lua_tonumber(L, 1);
	s->hide();
	return 0;
}

int scriptable::lua_onsignal(lua_State* L)
{
	int n=lua_gettop(L);
	ASSERT(n==2, "lua_onsignal bad params");
	scriptable* s=(scriptable*)lua_tonumber(L, 2);
	int sigs=lua_tonumber(L, 1);
	s->m_triggersignal=true;
	s->m_triggermask=sigs;
	return 0;
}

int scriptable::lua_raisesig(lua_State* L)
{
	int n=lua_gettop(L);
	ASSERT(n==2, "lua_raisesig bad params");
	scriptable* s=(scriptable*)lua_tonumber(L, 2);
	s->m_parent->raisesignal(lua_tonumber(L, 1));
	return 0;
}

int scriptable::lua_waitforsig(lua_State* L)
{
	int n=lua_gettop(L);
	ASSERT(n==2, "lua_waitforsig bad params");
	scriptable* t=(scriptable*)lua_tonumber(L, 1);
	t->m_waitforsignal=true;
	t->m_sigwaitingmask=lua_tonumber(L, 2);
	return 0;
}

//#define PROFILE_RANDOM

#ifdef PROFILE_RANDOM
TOOL_PROFILER_INIT;
#endif

int scriptable::lua_random (lua_State *L)
{
#ifdef PROFILE_RANDOM
	TOOL_PROFILER_START;
#endif
	int r = (int)rand();
	switch(lua_gettop(L))
	{  /* check number of arguments */
	case 0: {  /* no arguments */
			lua_pushnumber(L, r);  /* Number between 0 and RAND_MAX */
			break;
		}
	case 1: {  /* only upper limit */
			int u = lua_tonumber(L, 1);
			ASSERT(1<=u, "interval is empty");
			bios_div(r,u,&u);
			lua_pushnumber(L, (int)u+1);  /* integer between 1 and `u' */
			break;
		}
	case 2: {  /* lower and upper limits */
			int l = lua_tonumber(L, 1);
			int u = lua_tonumber(L, 2);
			ASSERT(l<=u, "interval is empty");
			bios_div(r,u-l+1,&u);
			lua_pushnumber(L, (int)(u)+l);	/* integer between `l' and `u' */
			break;
		}
	default: ASSERT(0, "wrong number of arguments");
	}
#ifdef PROFILE_RANDOM
	TOOL_PROFILER_END;
#endif
	return 1;
}

