// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#define SEARCH_WIDTH 1

#include <sprite.h>
#include <stage.h>
#include <misc.h>
#include <hardware.h>

//#define DEBUG_QUEUE

void astarqueue::nodepush(node* n)
{
	int qn=qnum(n->x,n->y);
#ifdef DEBUG_QUEUE
	uprintf("nodepushed on %x %x\n", (int)q,(int)n);
#endif
	n->qparent=queue[qn];
	queue[qn]=n;
}

node* astarqueue::nodepop()
{
	unsigned int c;
	unsigned int tlc=0xffffffff;
	node* tlcp=0;
	node* tlcpp=0;
	int tlcq=0;

	for(int i=0;i<A_QUEUESPLIT;i++)
	{
		node* last=0;
		node* qt=queue[i];

		while(qt)
		{
			c=qt->tc;
			if(c<tlc)
			{
				tlc=c;
				tlcp=qt;
				tlcpp=last;
				tlcq=i;
			}
			last=qt;
			qt=qt->qparent;
		}
	}

	if(tlcpp) tlcpp->qparent=tlcp->qparent;
	else queue[tlcq]=tlcp->qparent;
	return tlcp;
}

void astarqueue::removenode(u32 x, u32 y)
{
	node* qt=queue[qnum(x,y)];
	node* last=0;
	while(qt)
	{
		if(qt->x==x&&qt->y==y)
		{
			if(last)
				last->qparent=qt->qparent;
			else {
#ifdef DEBUG_QUEUE
				uprintf("oddball\n");
#endif
				queue[qnum(x,y)]=qt->qparent;
			}
			return;
		}
		last=qt;
		qt=qt->qparent;
	}
}

node::node(u32 xx, u32 yy)
{
#ifdef DEBUG_QUEUE
	uprintf("new node %x\n",(int)this);
#endif
	x=xx;
	y=yy;
	cfs=0;
	tc=0;
	parent=0;
	qparent=0;
}

/// Estimates the cost for travelling from one point to another.
static u32 estimate(u32 x, u32 y, u32 dx, u32 dy) MEM_FUNC_IN_IWRAM;
static u32 estimate(u32 x, u32 y, u32 dx, u32 dy)
{
	return(abs(x-dx)+abs(y-dy))<<2;	   // the shift overestimates the cost, reducing the number of iterations
}

/// Calculates the exact cost for travelling from a node to one of its neighbours.
#ifdef REAL_TRAVERSECOST
static u32 traversecost(u32 x, u32 y, u32 dx, u32 dy) MEM_FUNC_IN_IWRAM;
static u32 traversecost(u32 x, u32 y, u32 dx, u32 dy)
{
	int xx=x-dx;
	int yy=y-dy;
	if(xx && yy) return 3; // sqrt(2)*2
	if(xx || yy) return 2;
	return 0;
}
#else
#define traversecost(x, y, dx, dy) (1)	// the real value makes things slower (more iterations), which I don't like...
#endif

/// Checks whether a node for this position is already in the given queue.
node* astarqueue::isin(u32 x, u32 y)
{
	node* q=queue[qnum(x,y)];
	while(q)
	{
		if(q->x==x && q->y==y) return q;
		q=q->qparent;
	}
	return 0;
}

/// Checks if a sprite occupies the whole or a part of a certain tile.
static bool spriteoccupiestile(sprite* s, int x, int y) MEM_FUNC_IN_IWRAM;

static bool spriteoccupiestile(sprite* s, int x, int y)
{
	if(!(s->getboundxpos()>=x+8 || s->getboundxpos()+s->getboundxsize()<=x || s->getboundypos()>=y+8 ||
		 s->getboundypos()+s->getboundysize()<=y))
		return true;
	else
		return false;
}

//#define DEBUG_STEPISVALID

/// Checks whether going from one location to another is blocked by anything. Assumes that the distance traveled
/// is always 0 or 1 for each dimension. Ignores sprites. REMEMBERME: Does not work properly with bounding boxes
/// larger than one tile (pathological case).
bool sprite::stepisvalid(u32 x, u32 y, u32 dx, u32 dy)
{
	u32 row=0xffffffff;
	u32 col=0xffffffff;
	u32 endaddrow=0;
	u32 endaddcol=0;
	u32 startaddrow=0;
	u32 startaddcol=0;

	//ASSERT(x<(u32)m_parent->getxsize(), "x out of bounds");
	//ASSERT(y<(u32)m_parent->getysize(), "y out of bounds");

	if(dx<x) {
		endaddrow=1; col=0;
	}
	if(dx>x) {
		startaddrow=1; col=m_xsize/8-1;
	}
	if(dy<y) {
		endaddcol=1; row=0;
	}
	if(dy>y) {
		startaddcol=1; row=m_ysize/8-1;
	}

	if(row==0xffffffff || col==0xffffffff)
	{
		endaddrow=0;
		endaddcol=0;
		startaddrow=0;
		startaddcol=0;
	}

#ifdef DEBUG_STEPISVALID
	uprintf("x %d y %d dx %d dy %d row %d col %d ear %d eac %d sar %d sac %d\n",x,y,dx,dy, row, col,endaddrow,endaddcol,startaddrow,startaddcol);
#endif
	u32 i;
	u8 block;
	sprite** sprites;
	sprite* s;

	if(m_honorsprites)
		sprites=m_parent->getsprites();

	if(row!=0xffffffff)
	{
		for(i=dx-startaddrow;i<dx+m_xsize/8+endaddrow;i++)
		{
			if(m_honorsprites)
			{
				for(int j=0;j<m_parent->getspritecount();j++)
				{
					s=m_parent->getsprites()[j];
					if(s==this || !s->isvisible() || !s->spritecollisionenabled()) continue;
					if(spriteoccupiestile(s, i*8, (dy+row)*8))
						return false;
				}
			}
			block=m_parent->getmapdata(m_collidemap, i, dy+row);
			if((block&(UNOPT_BLOCK_MASK+UNOPT_WALKABLE_MASK))==UNOPT_BLOCK_MASK)	// block is there and not walkable
				return false;
		}
	}
	if(col!=0xffffffff)
	{
		for(i=dy-startaddcol;i<dy+m_ysize/8+endaddcol;i++)
		{
			if(m_honorsprites)
			{
				for(int j=0;j<m_parent->getspritecount();j++)
				{
					s=m_parent->getsprites()[j];
					if(s==this || !s->isvisible() || !s->spritecollisionenabled()) continue;
					if(spriteoccupiestile(s, (dx+col)*8, i*8))
						return false;
				}
			}
			block=m_parent->getmapdata(m_collidemap, dx+col, i);
			if((block&(UNOPT_BLOCK_MASK+UNOPT_WALKABLE_MASK))==UNOPT_BLOCK_MASK)	// block is there and not walkable
				return false;
		}
	}
	return true;
}

//#define PROFILE_PATHBUILDING
//#define DEBUG_ASTAR

#ifdef PROFILE_PATHBUILDING
TOOL_PROFILER_INIT;
#endif

/// Finds a path from one point on the collidemap to another. The initial call returns immediately. The cost of subsequent
/// calls depends on the setting of m_maxiter. The maxmimum amount of memory consumed depends on the setting of m_maxnodecount.
/// After it has finished it relies on on other functions to delete its queues.
u16* sprite::findpath(u32 x, u32 y, u32 dx, u32 dy)
{
	node* n;
	u16* retval=0;

	if(!m_resumepathing)	// initial setup
	{
		m_open=new astarqueue;
		m_closed=new astarqueue;
		m_nodecounter=0;

		m_pathxdest=dx;
		m_pathydest=dy;

		n=new node(x, y); m_nodecounter++;
		n->tc=estimate(x, y, dx, dy);
		m_open->nodepush(n);

		m_resumepathing=true;
		return 0;
	}

	// Profiling timer is running, was started by stage::compute()
	int time=hw::getproftimer();

	while(!m_open->isempty())
	{
		if(hw::getproftimer()-time>m_maxtime)	// exceeded our time contingent.
		{
#ifdef DEBUG_ASTAR
			uprintf("maxiter exceeded at %d\n",hw::getproftimer()-time);
#endif
			m_resumepathing=true;
			return 0;
		}

		n=m_open->nodepop();
#ifdef DEBUG_ASTAR
		//uprintf("nodepopped from %x %x\n", (int)m_open,(int)n);
#endif

		if(n->x==m_pathxdest && n->y==m_pathydest)	   // goal reached
		{
#ifdef PROFILE_PATHBUILDING
			TOOL_PROFILER_START;
#endif

			// count path nodes; this is not the same as m_nodecount!
			int count=0;
			node* nn=n;
			while(nn->parent)
			{
				count++;
				nn=nn->parent;
			}
#ifdef DEBUG_ASTAR
			uprintf("n counter: %d\n", count);
#endif

			// build the final path array
			u16* ns=new u16[count*2+2];
			node* sn=n;
			for(int i=count;i;i--)
			{
				ns[(i-1)*2]=n->x;
				ns[(i-1)*2+1]=n->y;
				n=n->parent;
			}
			ns[count*2]=0xffff;
			ns[count*2+1]=0xffff;

			delete sn;	// the node popped off the open queue.

#ifdef PROFILE_PATHBUILDING
			TOOL_PROFILER_END;
#endif
#ifdef DEBUG_ASTAR
			uprintf("a* count %d (time %d, nodecount %d)\n",count,hw::getproftimer()-time,m_nodecounter);
#endif

			retval=ns;
			goto finished;
		}
		else
		{
			// check all neighbouring locations
			for(int j=n->y-SEARCH_WIDTH;j<=n->y+SEARCH_WIDTH;j++)
				for(int i=n->x-SEARCH_WIDTH;i<=n->x+SEARCH_WIDTH;i++)
				{
					// check for out of bounds
					if(i<0 || j<0 || j>=m_parent->m_ysize || i>=m_parent->m_xsize) continue;

					// n is not a neighbour of itself
					if(i==n->x && j==n->y) continue;

					// check if we can actually go there
					if(!stepisvalid(n->x,n->y,i, j)) continue;

					bool isinclosed=false;
					bool isinopen=false;

					int newcost=n->cfs+traversecost(n->x,n->y,i,j);

					// check if this node is already in a list
					node* nn=0;
					nn=m_open->isin(i,j); if(nn)   isinopen=true;
					if(!nn)
					{
						nn=m_closed->isin(i,j);
						if(nn) isinclosed=true;
					}

					// skip if we already have a cheaper way there
					if(nn&&(nn->cfs<=newcost)) continue;

					if(!nn)
					{
						nn=new node(i, j);
						m_nodecounter++;
					}

					nn->parent=n;
					nn->cfs=newcost;
					nn->tc=newcost+estimate(i, j, m_pathxdest, m_pathydest);

					if(isinclosed)
						m_closed->removenode(i, j);

					if(isinopen)	// lists are not sorted, so if it's in open already, we're done.
						continue;

					m_open->nodepush(nn);
				}
		}
		m_closed->nodepush(n);

		if(m_nodecounter>m_maxnodecount) {
#ifdef DEBUG_ASTAR
			uprintf("max nodecount exceeded\n");
#endif
			goto finished;
		}
	}

#ifdef DEBUG_ASTAR
	uprintf("no path found to %d %d\n",m_pathxdest,m_pathydest);
#endif

	finished:	// Needs to be done whenever pathfinding terminates. Putting this here is not exactly pretty, but saves a few bytes.
	// IWRAM's limited, you know...
	m_resumepathing=false;
	m_nodecounter=0;
	m_deleteclosed=true;
	m_deleteopen=true;
	return retval;
}
