// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <stage.h>
#include <vbl.h>
#include <misc.h>
#include <hardware.h>

//#define DEBUG_STAGECTOR

stage::stage(int first_bg, const mapdata* playfield)
{
	int i,j,k;
	int temp1,temp2;

	ASSERT(playfield->num_bgs>0, "need to have at least one background");
	ASSERT(first_bg>=0 && first_bg <=3, "invalid first background");
	ASSERT(first_bg+playfield->num_bgs<=4, "max. 4 backgrounds");

	m_first_bg=first_bg;
	m_num_bgs=playfield->num_bgs;

	M_DISCNT_FRCBLK_ENA;	// CONSIDERME: Is there a better way to hide this shebang than blanking the whole screen?

	ham_SetBgMode(0);
	ham_LoadBGPal((void*)(playfield->colormap),playfield->colormapsize);

	// init all bgs to the same tileset
	ham_bg[first_bg].ti=ham_InitTileSet((void*)(playfield->tileset),playfield->tilesetsize,1,1);
	ham_bg[first_bg].mi=ham_InitMapEmptySet(MAP_MODE,0);
	for(i=first_bg+1;i<m_num_bgs+first_bg;i++)
	{
		ham_bg[i].ti=ham_bg[first_bg].ti;
		ham_bg[i].mi=ham_InitMapEmptySet(MAP_MODE,0);
	}

	// map fragment inserter copies 32 bits (2 tiles) at a time
	ASSERT((playfield->xsize&1)==0,"odd x map dimenson not allowed");

	m_xsize=playfield->xsize;
	m_ysize=playfield->ysize;

	m_rexcess=m_bexcess=m_lexcess=m_texcess=0;

	m_blockstr=(BLKSTR*)(playfield->blockstr);

	m_scrolllocked=-1;
	m_scrollborderx=m_scrollbordery=32;
	m_spritecnt=0;

	m_movedeltax=m_movedeltay=m_movecountdown=0;
	m_ismoving=false;

	m_sighted=m_sightedby=NULL;

	m_signals=0;
	m_signalsnext=0;

	m_ticks=0;

	m_radarbg=0;
	m_radarenable=false;
	m_radarx=m_radary=0;
	m_radarxsize=0;
	m_radarysize=0;
	m_radarpalette=0;

	m_numhighlights=0;
	m_highlightoldxpos=0;
	m_highlightoldypos=0;

	m_defaultcolmap=0;

	m_numcamhints=0;

	m_maxcomputetime=(90-10)*11*VBL_FRAMESKIP; // 90% of VBL_FRAMESKIP frames
	m_fixedprio=1;	// The first sprite (generally the player) is always king.

	// compute number of 1/4 screen map sections
	m_mapsectionsx=(m_xsize+MAP_SIZE_HALF-1)/MAP_SIZE_HALF;
	m_mapsectionsy=(m_ysize+MAP_SIZE_HALF-1)/MAP_SIZE_HALF;

	for(i=0;i<m_num_bgs;i++)
	{
		m_mapdata[i]=playfield->mapdata[i];
		m_mapdata_unopt[i]=playfield->mapdata_unopt[i];
	}

	// init map sections
	for(i=0;i<m_num_bgs;i++)
		m_map[i]=new map_fragment_info_ptr[m_mapsectionsx*m_mapsectionsy];

	for(j=0;j<m_mapsectionsy;j++)
		for(i=0;i<m_mapsectionsx;i++)
		{
			// size of map section
			temp1=m_xsize-MAP_SIZE_HALF*i; if(temp1>MAP_SIZE_HALF) temp1=MAP_SIZE_HALF;
			temp2=m_ysize-MAP_SIZE_HALF*j; if(temp2>MAP_SIZE_HALF) temp2=MAP_SIZE_HALF;

			for(k=0;k<m_num_bgs;k++)
			{
#ifdef DEBUG_STAGECTOR
				uprintf("mapdata[%d] %d\n",k,(int)(playfield->mapdata[k]));
#endif
				m_map[k][j*m_mapsectionsx+i]=ham_InitMapFragment((void*)(playfield->mapdata[k]),m_xsize,m_ysize,i*MAP_SIZE_HALF,j*MAP_SIZE_HALF,
																 temp1,temp2,0);
#ifdef DEBUG_STAGECTOR
				uprintf("m_map[%d][0]->src %d m_map[%d][%d]->src %d\n",k,(int)m_map[k][0]->src,k,j*m_mapsectionsx+i,(int)(m_map[k][j*m_mapsectionsx+i]->src));
#endif
			}
		}

	// init currently displayed map sections to "undefined"
	m_currentfrag[0]=m_currentfrag[1]=m_currentfrag[2]=m_currentfrag[3]=0xffff;

	// init map inversion to false
	m_mapinvert[0]=m_mapinvert[1]=m_mapinvert[2]=m_mapinvert[3]=false;

#ifdef DEBUG_STAGECTOR
	uprintf("m_map[0][0]->src %d\n",(int)m_map[0][0]->src);
#endif

	// BGs need to be set up before setpos is called (map inserter reads BG?CNT regs)
	for(i=first_bg;i<first_bg+m_num_bgs;i++)
	{
#ifdef DEBUG_STAGECTOR
		uprintf("init bg %d\n", i);
#endif
		ham_InitBg(i,1,3-i,0);
	}

	setpos(0,0);

	M_DISCNT_FRCBLK_DIS;

	vbl::instance()->addhandler(this);
}

stage::~stage()
{
	int i;

	vbl::instance()->removehandler(this);

	M_DISCNT_FRCBLK_ENA;	// CONSIDERME: Is there a better way to hide this shebang than blanking the whole screen?

	for(i=0;i<m_num_bgs;i++)
	{
		// We assign the same tileset to every BG, so we can only use ham_DeInitBg() once.
		if(i==0)
			ham_DeInitBg(m_first_bg+i);
		else
		{
			ham_DeInitMapSet(ham_bg[m_first_bg+i].mi);
			M_DISCNT_BGX_SET(m_first_bg+i,0);	// disable BG
		}

		delete[] m_map[i];
	}

	deleteallsprites();

	if(m_radarenable) enableradar(false);

	// Delete all A* pathfinding queues. See sprite::~sprite() for an explanation why this is done here.
	hw::startproftimer();
	// The argument is larger than any value the 16-bit timer will ever have, so deletethem() will delete until all queues are gone.
	while(astarqueue::deletethem(0x10000));
	hw::stopproftimer();

	M_DISCNT_FRCBLK_DIS;
}

#define DEBUG_DUMPPROFILES

void stage::deleteallsprites()
{
#ifdef DEBUG_DUMPPROFILES
	uprintf("Sprite profiles:\n");
	for(int i=0; i<getspritecount();i++)
		uprintf("%s at 0x%x: %d ticks\n", getsprites()[i]->getdescription(), (int)getsprites()[i], getspriteprofile(i));
#endif

	hidehighlights();

	// deleting a sprite modifies m_spritecnt and the sprite list, so we need to
	// delete the first sprite until there are no more left.
	while(m_spritecnt>0)
		delete m_sprites[0];

	spam_CopyObjToOAM();
}

void stage::vbl_handler()
{
	for(int i=0; i<m_spritecnt; i++)
		m_sprites[i]->vbl_handler();

	if(m_ismoving) moveme();	// moving a stage does not change the game's world.

	spam_CopyObjToOAM();

	for(int i=0; i<m_num_bgs; i++)
		ham_SetBgXY(m_first_bg+i, m_xpos, m_ypos);
}

void stage::addsprite(sprite* sprite)
{
	VBL_SUSPEND(true);
	m_sprites[m_spritecnt]=sprite;
	m_spriteprofile[m_spritecnt++]=0;
	VBL_SUSPEND(false);
	sprite->setcollidemapno(m_defaultcolmap);
}

void stage::deletesprite(sprite* obj)
{
	for(int i=0; i<m_spritecnt; i++)
	{
		if(m_sprites[i]==obj)
		{
			VBL_SUSPEND(true);
			m_spritecnt--;
			for(int j=i;j<m_spritecnt;j++)
			{
				m_sprites[j]=m_sprites[j+1];
				m_spriteprofile[j]=m_spriteprofile[j+1];
			}
			VBL_SUSPEND(false);
			break;
		}
	}
}

//#define DEBUG_SPRITEMOVED

u32 stage::spritemoved(sprite* obj, int oldx, int oldy)
{
	int newx, newy;

	if(obj->isscreenframeofreference())	return 0xffffffff; // A-OK!

	int adjust = obj->getadjust();
	int adjustspeed=obj->getadjustspeed();
	int cm=obj->getcollidemapno();

	if(oldx!=0x7fff			// no collision check necessary
	   && obj->getmapcollision())
	{
		// convert pixel to tiles
		u32 left=obj->getboundxpos()/8;
		u32 top=obj->getboundypos()/8;
		u32 right=(obj->getboundxpos()+obj->getboundxsize()-1)/8;
		u32 bottom=(obj->getboundypos()+obj->getboundysize()-1)/8;

#ifdef DEBUG_SPRITEMOVED
		uprintf("collide? l %d r %d t %d b %d\n",left,right,top,bottom);
#endif
		if(checkcollide(cm,left,right,top,bottom))
		{
#if 1

			// check if adjusting helps
			if(adjust != 0xff)
			{
				for(int j=0; j<adjust;j++)
					if(oldy==obj->getboundypos())
					{
						//uprintf("adjust y\n");
						if(oldy+j < m_ysize*8 && !checkcollide(cm,left,right,(oldy+j)/8,(oldy+obj->getboundysize()-1+j)/8))
							return oldx+((oldy+adjustspeed)<<16);	// FIXME: "1" should be adjustspeed or sth like that (also below)
						else if(oldy-j >= 0 && !checkcollide(cm,left,right,(obj->getboundypos()-j)/8,(oldy+obj->getboundysize()-1-j)/8))
							return oldx+((oldy-adjustspeed)<<16);
					}
					else if(oldx==obj->getboundxpos())
					{
						//uprintf("adjust x\n");
						if(oldx+j < m_xsize*8 && !checkcollide(cm,(oldx+j)/8,(oldx+obj->getboundxsize()-1+j)/8,top,bottom))
							return oldx+adjustspeed+(oldy<<16);
						else if(oldx-j >= 0 && !checkcollide(cm,(oldx-j)/8,(oldx+obj->getboundxsize()-1-j)/8,top,bottom))
							return oldx-adjustspeed+(oldy<<16);
					}
			}
#endif
#if 1
			// check if shortening the change in one direction helps
			// x
			int xx=(oldx+SGN(obj->getboundxpos()-oldx));
			if(!checkcollide(cm, xx/8, (xx+obj->getboundxsize()-1)/8,top,bottom))
				return xx+(obj->getboundypos()<<16);
			// y
			int yy=(oldy+SGN(obj->getboundypos()-oldy));
			if(!checkcollide(cm, left, right, yy/8, (yy+obj->getboundysize()-1)/8))
				return obj->getboundxpos()+(yy<<16);
#endif
#if 1

			// check if omitting the change in one direction helps
			// x
			if(!checkcollide(cm,oldx/8,(oldx+obj->getboundxsize()-1)/8,top,bottom))
				return oldx+(obj->getboundypos()<<16);
			// y
			if(!checkcollide(cm,left,right,oldy/8,(oldy+obj->getboundypos()-1)/8))
				return obj->getboundxpos()+(oldy<<16);
#endif


			return oldx+(oldy<<16);
		}
	}

	if(m_scrolllocked != obj->m_objno) return 0xffffffff;	// A-OK!

	// If we get here we need to lock the camera to this sprite.

	newx=m_xpos;
	newy=m_ypos;

	// Check if we need to follow a camera hint.
	bool camhint=false;
	if(m_numcamhints)
	{
		int o_l=obj->getboundxpos();
		int o_t=obj->getboundypos();
		int o_r=o_l+obj->getboundxsize();
		int o_b=o_t+obj->getboundysize();

		for(int i=0; i<m_numcamhints; i++)
		{
			if(overlap(o_l,o_t,o_r,o_b,m_camhints[i].l,m_camhints[i].t,m_camhints[i].r,m_camhints[i].b))
			{
				newx=m_camhints[i].camx;
				newy=m_camhints[i].camy;
				camhint=true;
				break;
			}
		}
	}

	// No camera hint -> obey the scroll border settings.
	if(!camhint)
	{
		if(obj->getxpos()>m_xpos+SCREEN_WIDTH-obj->getxsize()-m_scrollborderx)
			newx=obj->getxpos()-SCREEN_WIDTH+obj->getxsize()+m_scrollborderx;
		if(obj->getxpos()<m_xpos+m_scrollborderx)
			newx=obj->getxpos()-m_scrollborderx;
		if(obj->getypos()>m_ypos+SCREEN_HEIGHT-obj->getysize()-m_scrollbordery)
			newy=obj->getypos()-SCREEN_HEIGHT+obj->getysize()+m_scrollbordery;
		if(obj->getypos()<m_ypos+m_scrollbordery)
			newy=obj->getypos()-m_scrollbordery;
	}

	fixpos(&newx, &newy);

	if(newx != m_xpos || newy != m_ypos)
	{
		stop();
		
		// Smoothen sudden movements by using moveto().
		if(abs(newx-m_xpos)>6 || abs(newy-m_ypos)>6)
			moveto(newx, newy,2,1);
		else
			setpos(newx,newy);
	}

	return 0xffffffff;	// A-OK!
}

void stage::scrolllock(sprite* obj)
{
	m_scrolllocked=obj->m_objno;
	spritemoved(obj,0x7fff,0x7fff);
}

void stage::addcamhint(u16 l, u16 t, u16 r, u16 b, u16 camx, u16 camy)
{
	camhint* m=&m_camhints[m_numcamhints];
	m->l=l; m->t=t; m->r=r; m->b=b; m->camx=camx; m->camy=camy;
	m_numcamhints++;
}

#define DEBUG_SCHEDULING
//#define PROFILE_RESCHEDULING
//#define DEBUG_RESCHEDULING
#define DEBUG_LOWTIME

#ifdef PROFILE_RESCHEDULING
TOOL_PROFILER_INIT;
#endif

//#define PROFILE_RADAR

#ifdef PROFILE_RADAR
TOOL_PROFILER_INIT;
#endif

void stage::compute(bool lowtime)
{
	hw::initproftimer();
	hw::resetproftimer();
	hw::startproftimer();

	m_ticks+=VBL_FRAMESKIP;

	bool m_spriteexecuted[m_spritecnt];
	bool m_spriteexecutedlowtime[m_spritecnt];
	for(int i=0;i<m_spritecnt;i++)
	{
		m_sprites[i]->m_hasmovedlastturn=m_sprites[i]->m_hasmoved;
		m_sprites[i]->m_hasmoved=false;
		m_spriteexecuted[i]=m_spriteexecutedlowtime[i]=false;
	}

#ifdef PROFILE_RADAR
	TOOL_PROFILER_START;
#endif
	if(m_radarenable && !(m_ticks&3)) drawradar(); // draw radar every 4 ticks
#ifdef PROFILE_RADAR
	TOOL_PROFILER_END;
#endif

	m_signalsnext=0;

	for(int i=0;i<m_spritecnt;i++)
	{
		int time=hw::getproftimer();
		if(!m_spriteexecuted[i] && !m_spriteexecutedlowtime[i])
		{
			if(m_maxcomputetime-time>m_spriteprofile[i])
			{
				m_sprites[i]->compute(false);
				if(hw::getproftimer()-time>m_spriteprofile[i]) m_spriteprofile[i]=hw::getproftimer()-time;
				if(m_spriteprofile[i]>=m_maxcomputetime)
				{
#ifdef DEBUG_SCHEDULING
					uprintf("MEGAWARNING: sprite %s at %x takes more than maxcomputetime! %d ticks!!\n", m_sprites[i]->getdescription(), m_sprites[i],m_spriteprofile[i]);
#endif
					m_spriteprofile[i]=m_maxcomputetime-(m_maxcomputetime/8);
				}
				m_spriteexecuted[i]=true;
			}
			else if(!m_spriteexecutedlowtime[i] && m_maxcomputetime-time>50)
			{
				m_sprites[i]->compute(true);
				m_spriteexecutedlowtime[i]=true;
#ifdef DEBUG_LOWTIME
				if(hw::getproftimer()-time>50) uprintf("--- WARNING: %s at %x exceeded max lowtime by %d\n",
													   m_sprites[i]->getdescription(), m_sprites[i], hw::getproftimer()-time-50);
#endif				
			}
		}
	}

	m_signals=m_signalsnext;

#ifdef DEBUG_SCHEDULING

	// Given enough time all sprites should have been executed either fully or with lowtime.
	if(hw::getproftimer()<m_maxcomputetime-50)
		for(int i=0;i<m_spritecnt;i++)
			if(!(m_spriteexecuted[i]||m_spriteexecutedlowtime[i])) uprintf("Warning: sprite %s at %x not executed\n", m_sprites[i]->getdescription(),m_sprites[i]);
#endif

#ifdef PROFILE_RESCHEDULING
	TOOL_PROFILER_START;
#endif

	// Give those not executed this time a better chance next time.
	int sortcnt=m_fixedprio;	// where to insert the upscheduled sprites
	for(int i=m_fixedprio; i<m_spritecnt; i++)
		if(!m_spriteexecuted[i])
			switchpriority(i,sortcnt++);

#ifdef PROFILE_RESCHEDULING
	TOOL_PROFILER_END;
#endif

	int dtime;
	if((dtime=m_maxcomputetime-hw::getproftimer()-30)>0)
		astarqueue::deletethem(dtime);

	hw::stopproftimer();

#ifdef DEBUG_SCHEDULING
	static int maxticks=0;
	if(hw::getproftimer()>maxticks)
	{
		maxticks=hw::getproftimer();
		uprintf("took me %d ticks\n", maxticks);
	}
	if(hw::getproftimer()>1100*VBL_FRAMESKIP)
		uprintf("WARNING: %d frame(s) exceeded by %d ticks!\n",VBL_FRAMESKIP,hw::getproftimer()-1100*VBL_FRAMESKIP);
	else if(hw::getproftimer()>m_maxcomputetime+110*VBL_FRAMESKIP)
		uprintf("Warning: time + %d exceeded by %d ticks!\n",110*VBL_FRAMESKIP,hw::getproftimer()-m_maxcomputetime-110*VBL_FRAMESKIP);
#endif

	// object::compute() is not called here, moveme() is handled in vbl_handler_method() because moving a stage does not change
	// the game world, it is purely cosmetic.
}

void stage::switchpriority(int first, int second)
{
	//ASSERT(first<m_spritecnt && second<m_spritecnt, "fuck");
	VBL_SUSPEND(true);
	sprite* s=m_sprites[first];
	int p=m_spriteprofile[first];
#ifdef DEBUG_RESCHEDULING
	uprintf("switching %d (%s) and %d (%s) at count %d (fp %d)\n", first,s->getdescription(), second,m_sprites[second]->getdescription(), m_spritecnt, m_fixedprio);
#endif
	m_spriteprofile[first]=m_spriteprofile[second];
	m_spriteprofile[second]=p;
	m_sprites[first]=m_sprites[second];
	m_sprites[second]=s;
	VBL_SUSPEND(false);
}

void stage::setprofiletime(sprite* s, int time)
{
	for(int i=0; i<m_spritecnt; i++)
		if(m_sprites[i]==s)
		{
			m_spriteprofile[i]=time*11;
			break;
		}
}

void stage::fixpos(int* x, int* y)
{
	if(*x<m_lexcess*8) *x=m_lexcess*8;
	if(*y<m_texcess*8) *y=m_texcess*8;
	if(*x>(m_xsize-m_rexcess)*8-SCREEN_WIDTH) *x=(m_xsize-m_rexcess)*8-SCREEN_WIDTH;
	if(*y>(m_ysize-m_bexcess)*8-SCREEN_HEIGHT) *y=(m_ysize-m_bexcess)*8-SCREEN_HEIGHT;
}

//#define DEBUG_SETPOS

/// This method takes about 3 scanlines per map insertion (profiled on VBA, real GBA may vary).
/// You probably don't have to check the time used by it to ensure smooth operation.
void stage::setpos(int x, int y)
{
	int needpos,i,j;
	int relx,rely;	// relevant position
	int mapx,mapy;	// position of new map section on map
	int memoffset[4];

	fixpos(&x, &y);

	m_xpos=x;
	m_ypos=y;

	// check validity of current map section on the four screen corners
	for(i=0;i<4;i++)
	{
		// plan 4 pixels ahead in every direction
		if(i&1)	relx=m_xpos+SCREEN_WIDTH+4;
		else relx=m_xpos>4?m_xpos-4:m_xpos;
		if(i&2)	rely=m_ypos+SCREEN_HEIGHT+4;
		else rely=m_ypos>4?m_ypos-4:m_ypos;

		// which map section do we need?
		needpos=relx/MAP_SIZE_HP+rely/MAP_SIZE_HP*m_mapsectionsx;

		// convert to tiles, modulo hw-map size, align at beginning of hw-map section
		mapx=((relx/8)&(MAP_SIZE-1))&(~(MAP_SIZE_HALF-1));
		mapy=((rely/8)&(MAP_SIZE-1))&(~(MAP_SIZE_HALF-1));


		// offsets of appropriate hw-map section for all bgs
		for(j=m_first_bg;j<m_num_bgs+m_first_bg;j++)
		{
			volatile u16* R_BGiCNT=(u16*)(MEM_IO+8+j*2);
			memoffset[j-m_first_bg]=(mapx>>5)*1024+(mapy>>5)*2048+(((*R_BGiCNT)&0x1f00)<<2);
#ifdef DEBUG_SETPOS
			uprintf("memoffset[%d] %d\n",j-m_first_bg,memoffset[j-m_first_bg]);
#endif
		}

		if(m_currentfrag[(mapx>>5)+(mapy>>4)]!=needpos)
		{
#ifdef DEBUG_SETPOS
			uprintf("i %d needpos %d xpos %d ypos %d relx %d rely %d mapx %d mapy %d\n",i,needpos,xpos,ypos,relx,rely,mapx,mapy);
#endif


			// my own map fragment inserter; with a DMA-optimized map it is more than 50 times faster than
			// ham_InsertMapFragment(). Without it, it's still substantially faster than HAM (used to be factor 20,
			// introducing a variable number of bgs slowed it down to about 5.6, adding the switch
			// statements got it up to 8.75 again.)
			for(j=0;j<m_num_bgs;j++)
			{
				VBL_SUSPEND(true);		// handlers might use DMA3

				TOOL_DMA3_SET((m_mapdata[j]+needpos*1024),(u16*)0x6000000+memoffset[j],32*32/2,1,DMA_STARTAT_NOW);

				// newlib's memcpy does not work coz it copies bytewise, which is undefined when writing to GBA VRAM
				//memcpy((u16*)0x6000000+memoffset[j],(m_mapdata[j]+needpos*1024),32*32*2);
				VBL_SUSPEND(false);
			}
			m_currentfrag[(mapx>>5)+(mapy>>4)]=needpos;
		}
	}
	for(i=0;i<m_spritecnt;i++)
		m_sprites[i]->reposition();
}

void stage::setmapinversion(int map, bool invert)
{
	ASSERT(map<m_num_bgs, "cannot invert undefined map");
	m_mapinvert[map]=invert;
}

static const u16 radardefpal_raw[16]={COLOR_NAVY,COLOR_BLACK,/* green */ 0x18<<5,COLOR_YELLOW, COLOR_RED, COLOR_BLUE, COLOR_PURPLE, COLOR_MAROON};
const palettedata stage::m_radardefpal={ radardefpal_raw, 16};

void stage::setradar(int bg, int x, int y, int xsize, int ysize, const palettedata* pal)
{
	ASSERT(!((xsize&7)||(ysize&7)), "radar size must be multiple of 8");

	m_radarx=x;
	m_radary=y;
	m_radarxsize=xsize;
	m_radarysize=ysize;
	m_radarpalette=pal;
	m_radarbg=bg;
}

void stage::enableradar(bool enable)
{
	ASSERT(m_radarpalette, "radar not initialized yet!");
	if(enable==m_radarenable) return;
	if(enable)
	{
		ham_LoadBGPal16((void*)(m_radarpalette->palette), 15);

		ham_bg[m_radarbg].ti=ham_InitTileEmptySet(m_radarxsize/8*m_radarysize/8, 0, 0);
		ham_bg[m_radarbg].mi=ham_InitMapEmptySet(0, 0);

		// clear map
		ham_ReloadTileGfx(ham_bg[m_radarbg].ti, (u16*)transparent_tile, m_radarxsize/8*8, 1);
		for(int i=0;i<32;i++)
			for(int j=0;j<32;j++)
				spam_SetMapTileEx(m_radarbg, i, j, m_radarxsize/8*8, 0, 0, 15);

		// init used portion of map
		for(int i=0;i<m_radarxsize/8;i++)
			for(int j=0;j<m_radarysize/8;j++)
				spam_SetMapTileEx(m_radarbg, i, j, i+j*m_radarxsize/8, 0, 0, 15);

		ham_SetFxMode(1<<m_radarbg,BIT0|BIT1|BIT2|BIT3|BIT4,FX_MODE_ALPHABLEND);
		ham_SetFxAlphaLevel(15, 10);

		drawradar();
		m_radarenable=true;
		ham_SetBgXY(m_radarbg, -m_radarx, -m_radary);
		ham_InitBg(m_radarbg, 1, 0, 0);
	}
	else
	{
		ham_SetFxMode(0,0,FX_MODE_OFF);

		ham_DeInitBg(m_radarbg);
		m_radarenable=false;
	}
}

//#define DEBUG_OBJWINDOW

void stage::addhighlight(int x, int y, int centerx, int centery, const spritedata* bitmap)
{
	ASSERT(m_numhighlights<S_MAXHIGHLIGHTS-1, "max number of highlights exceeded");
	m_highlightsprite[m_numhighlights]=new sprite(this, bitmap, x-bitmap->xsize/2, y-bitmap->ysize/2);
	m_highlightsprite[m_numhighlights]->setdescription(NULL);

	// ham_SetObjMode() is totally broken.
	spam_SetObjMode(m_highlightsprite[m_numhighlights]->m_objno, OBJ_MODE_OBJWINDOW);
#ifdef DEBUG_OBJWINDOW
	uprintf("objatr0 %x\n", ham_obj[m_highlightsprite->m_objno].objatr0);
#endif

	// ham_CreateWin() is broken. What a surprise...
	// It ignores the outside_on_for param.
	spam_CreateWin(2, 0, 0, 0, 0, WIN_BG0|WIN_BG1|WIN_BG2|WIN_BG3|WIN_OBJ, WIN_BG0|WIN_BG1|WIN_BG2|WIN_BG3|WIN_OBJ, 1);

#ifdef DEBUG_OBJWINDOW
	uprintf("winout %x\n", R_WINOUT);
#endif

	ham_SetFxMode(BIT1|BIT2|BIT4, 0, FX_MODE_DARKEN);
	ham_SetFxBrightnessLevel(8);

	if(!m_numhighlights)
	{
		m_highlightoldxpos=m_xpos;
		m_highlightoldypos=m_ypos;
	}
	m_numhighlights++;

	moveto(x-centerx, y-centery, 2, 1);
}

void stage::addhighlightsprite(sprite* s, int centerx, int centery)
{
	addhighlight(s->getboundcenterx(), s->getboundcentery(), centerx, centery);
}

void stage::hidehighlights()
{
	while(m_numhighlights)
	{
		m_numhighlights--;
		delete m_highlightsprite[m_numhighlights];
	}
	ham_DeleteWin(2);
	ham_SetFxMode(0, 0, FX_MODE_OFF);

	moveto(m_highlightoldxpos,m_highlightoldypos,2,1);
}

void stage::replacemapdata(const mapdata* newmap)
{
	ASSERT(newmap->xsize == m_xsize && newmap->ysize == m_ysize && newmap->num_bgs == m_num_bgs, "new map has wrong dimensions");

	for(int i=0;i<m_num_bgs;i++)
	{
		m_mapdata[i]=newmap->mapdata[i];
		m_mapdata_unopt[i]=newmap->mapdata_unopt[i];
	}

	// redraw map
	m_currentfrag[0]=m_currentfrag[1]=m_currentfrag[2]=m_currentfrag[3]=0xffff;
	setpos(m_xpos, m_ypos);
}

void stage::action(sprite* me, sprite* them)
{
	if(them) them->action(me);
}

void stage::push(sprite* me, sprite* them)
{
	//uprintf("push thru stage\n");
	them->push(me);
}

void stage::setspritepalette(const palettedata* palette)
{
	sprite::setpalette(palette);
}

void stage::setdefaultcollidemap(int colmap)
{
	ASSERT(colmap<m_num_bgs, "invalid default collide map");
	m_defaultcolmap=colmap;
}

