// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <stage.h>
#include <vbl.h>
#include <misc.h>

//#define PROFILE_RADAR
//#define DEBUG_RADAR

#ifdef PROFILE_RADAR
TOOL_PROFILER_INIT;
#endif

// makes radar ~10% faster at the expense of code size
//#define SPACE_SPEED_TRADEOFF

void stage::drawradar()
{
#ifdef PROFILE_RADAR
	TOOL_PROFILER_START;
#endif
	u32 value;	// final value (4 pixels) to write to VRAM
	u8* dest=(u8*)MEM_HAMBG_SLOT_PTR(ham_bg[m_radarbg].ti->first_block);	// address of tile data for our BG

	int rx=m_xsize-m_rexcess-m_lexcess;	// radar x size
	int ry=m_ysize-m_bexcess-m_texcess;	// radar y size

#ifdef DEBUG_RADAR
	uprintf("rx %d ry %d\n", rx, ry);
#endif

	int xstart=(m_xpos+SCREEN_WIDTH/2)/8-m_radarxsize/2;	// map section to be displayed starts here
	int ystart=(m_ypos+SCREEN_HEIGHT/2)/8-m_radarysize/2;

	if(m_radarxsize>rx)
		xstart=rx/2-m_radarxsize/2;	// center in x direction
	else
	{
		if(xstart+m_radarxsize>rx+m_lexcess+1) xstart=rx+m_lexcess-m_radarxsize+1; // don't scroll beyond end of map
		if(xstart<m_lexcess) xstart=m_lexcess;	   // don't scroll before start of map
	}
	if(m_radarysize>ry)
		ystart=ry/2-m_radarysize/2;
	else
	{
		if(ystart+m_radarysize>ry+m_bexcess+1) ystart=ry+m_bexcess-m_radarysize+1;
		if(ystart<m_texcess) ystart=m_texcess;
	}

	int s;
	int offset;

	rx+=m_lexcess;
	ry+=m_texcess;

	// fill the tiles in our BG with proper data
	for(int i=0; i<m_radarxsize; i+=8)	   // iterate in x direction, in tile size steps
		for(int j=0; j<m_radarysize; j+=8) // iterate in y direction, in tile size steps
		{
			s=i+xstart;		// where to start on the map (x)
			for(int l=0;l<8;l++)   // iterate over lines of tile
			{
				value=0x11111111;	// all pixels black
				if(!(s>=rx || s<m_lexcess-8 || j+l+ystart>=ry || j+l+ystart<m_texcess)) // check if we're off the map
				{
					offset=XYTOMAP(s,j+l+ystart,m_xsize);	// where to read from

					// placing this here instead of outside the loops makes things a bit faster
					const u8* tmp=MAPDATA[1]+offset;	// FIXME: make the radar map configurable

					// 16-color mode, each nibble is one pixel
						switch(m_lexcess-s)		// skip blocks left of m_lexcess
						{
						default:
							if((tmp[0]&(UNOPT_BLOCK_MASK+UNOPT_INVISIBLE_MASK))==UNOPT_BLOCK_MASK) value+=1<<0;
						case 1:
							if(s+1<rx) if((tmp[1]&(UNOPT_BLOCK_MASK+UNOPT_INVISIBLE_MASK))==UNOPT_BLOCK_MASK) value+=1<<4;
						case 2:
							if(s+2<rx) if((tmp[2]&(UNOPT_BLOCK_MASK+UNOPT_INVISIBLE_MASK))==UNOPT_BLOCK_MASK) value+=1<<8;
						case 3:
							if(s+3<rx) if((tmp[3]&(UNOPT_BLOCK_MASK+UNOPT_INVISIBLE_MASK))==UNOPT_BLOCK_MASK) value+=1<<12;
						case 4:
							if(s+4<rx) if((tmp[4]&(UNOPT_BLOCK_MASK+UNOPT_INVISIBLE_MASK))==UNOPT_BLOCK_MASK) value+=1<<16;
						case 5:
							if(s+5<rx) if((tmp[5]&(UNOPT_BLOCK_MASK+UNOPT_INVISIBLE_MASK))==UNOPT_BLOCK_MASK) value+=1<<20;
						case 6:
							if(s+6<rx) if((tmp[6]&(UNOPT_BLOCK_MASK+UNOPT_INVISIBLE_MASK))==UNOPT_BLOCK_MASK) value+=1<<24;
						case 7:
							if(s+7<rx) if((tmp[7]&(UNOPT_BLOCK_MASK+UNOPT_INVISIBLE_MASK))==UNOPT_BLOCK_MASK) value+=1<<28;
						}
				}
				*(u32*)(&dest[(i/8+(j/8)*(m_radarxsize/8))*32+l*4])=value;
			}
		}

	for(int i=0; i<m_spritecnt;i++)
	{
		int w=m_sprites[i]->m_xsize/8;	// width of blob
		int h=m_sprites[i]->m_ysize/8;	// height of blob

		int c=m_sprites[i]->isvisible()?m_sprites[i]->m_radarcolor:0;
		if(!c) continue;				// zero equals invisible

		int x=(m_sprites[i]->m_xpos/8-xstart);
		int y=(m_sprites[i]->m_ypos/8-ystart);

		if(x<m_lexcess||y<m_lexcess||x>rx-xstart||y>ry-ystart) continue;   // out of current radar section

		// once for each tile the sprite has
		for(int j=0;j<h;j++)
			for(int k=0;k<w;k++)
			{
				int xx=x+k;
				int yy=y+j;
				offset=(xx/8+(yy/8)*(m_radarxsize/8))*32+(xx&7)/2+(yy&7)*4;	// conversion coords -> tile data offset

				// We can only write 16 bits (4 pixels) at a time...
				*(u16*)&dest[offset&(~1)]&=(0xfff0ffff<<((xx&3)*4))>>16;	// zero the nibble to modify
				*(u16*)&dest[offset&(~1)]|=c<<((xx&3)*4);				// set it to new value
			}
	}

#ifdef PROFILE_RADAR
	TOOL_PROFILER_END;
#endif
}

