// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

// BMP file color exchanger. Use this on BMP files that don't have their background at palette index 0.
// Takes two color indices as arguments, reads from stdin, writes to stdout.

#include <stdio.h>
#include <assert.h>

#pragma pack(1)
typedef struct
{
	short bfType;
	int bfSize;
	short bfReserved1;
	short bfReserved2;
	int bfOffBits;
} BITMAPFILEHEADER;

typedef struct
{
	int biSize;
	int biWidth;
	int biHeight;
	short biPlanes;
	short biBitCount;
	int biCompression;
	int biSizeImage;
	int biXPelsPerMeter;
	int biYPelsPerMeter;
	int biClrUsed;
	int biClrImportant;
} BITMAPINFOHEADER;

// REMEMBERME: This is not endianness-clean.
typedef union
{
	struct
	{
		char rgbBlue;
		char rgbGreen;
		char rgbRed;
		char rgbReserved;
	};
	int color;
} RGBQUAD;

int main(int argc, char** argv)
{
	int first,second;
	int tmp;
	int i;
	RGBQUAD colors[256];
	unsigned char rest[256<<10];	// 256k
	int restlength;
	BITMAPFILEHEADER fh;
	BITMAPINFOHEADER ih;

	first=atoi(argv[1]);
	second=atoi(argv[2]);

	fread(&fh, sizeof(BITMAPFILEHEADER), 1, stdin);
	fread(&ih, sizeof(BITMAPINFOHEADER), 1, stdin);
	assert(ih.biBitCount==8 || ih.biBitCount==4);
	fread(colors, sizeof(RGBQUAD), 1<<ih.biBitCount, stdin);
	tmp=colors[first].color;
	colors[first].color=colors[second].color;
	colors[second].color=tmp;
	restlength=fread(rest, 1, 256<<10, stdin);
	for(i=0; i<restlength; i++)
	{
		if(rest[i]==first) rest[i]=second;
		else if(rest[i]==second) rest[i]=first;
	}
	fwrite(&fh, sizeof(BITMAPFILEHEADER), 1, stdout);
	fwrite(&ih, sizeof(BITMAPINFOHEADER), 1, stdout);
	fwrite(colors, sizeof(RGBQUAD), 1<<ih.biBitCount, stdout);
	fwrite(rest, 1, restlength, stdout);
}
