// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

// Mappy GBA export file optimizer
// removes unused palette entries and optionally rearranges map data in a DMA-friendly way

// Defines:
// -DMAx: enable maparrayx[]
// -DMAPFILE=\"whatever.TXT\": use Mappy GBA export file whatever.TXT
// of the map for purposes of pathfinding and collision detection.

// 1st cmdline parameter: identifier prefix
// 2nd cmdline parameter: number of maps (must correspond to -DMAx defines)

#include <stdio.h>
#include <gbamappy.h>
#include MAPFILE

int main(int argc, char** argv)
{
	int blkstrsize;
	int i,j,k,l,m;
	int mx,my;
	short* maparray;
	int cmapremap[256];
	int palcount=0;
	int remap;

	printf("#include <mygba.h>\n");
	printf("#include <formats.h>\n");

	blkstrsize=sizeof(blockstr)/sizeof(BLKSTR);
	printf("const BLKSTR %s_blockstr[%d] __attribute__ ((aligned (4))) = {\n",argv[1],blkstrsize);
	for(i=0;i<blkstrsize;i++)
	{
		printf("{ %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, 0x%02x },\n",
			   blockstr[i].bgoff,blockstr[i].fgoff,blockstr[i].fgoff2,blockstr[i].fgoff3,blockstr[i].user1,blockstr[i].user2,blockstr[i].user3,
			   blockstr[i].user4,blockstr[i].user5,blockstr[i].user6,blockstr[i].user7,blockstr[i].bits);
	}
	printf("};\n");

	mx=sizeof(maparray0[0])/2;
	my=sizeof(maparray0)/mx/2;
	
	for(i=0;i<atoi(argv[2]);i++)
	{
		switch(i)
		{
#ifdef MA0
		case 0: maparray=(short*)maparray0; break;
#endif
#ifdef MA1
		case 1: maparray=(short*)maparray1; break;
#endif
#ifdef MA2
		case 2: maparray=(short*)maparray2; break;
#endif
#ifdef MA3
		case 3: maparray=(short*)maparray3; break;
#endif
		default:
			exit(1);
		}
		printf("const u16 %s_maparray%d[%d] __attribute__ ((aligned (4))) = {\n",argv[1],i,my*mx);
		if(mx%32 || my%32)
		{
			fprintf(stderr, "map dimensions not multiples of 32\n");
			exit(3);
		}
		for(j=0;j<my/32;j++)
		{
			for(k=0;k<mx/32;k++)
			{
				 for(l=0;l<32;l++)
				 {
					for(m=0;m<32;m++)
					{
						if(m && !(m%16)) printf("\n");
						printf("%d, ", blockstr[maparray[(l+j*32)*mx+m+k*32]].bgoff);
					}
					printf("\n");
				 }
			}
		}
		printf("};\n");
		printf("const u8 %s_maparray%d_plain[%d][%d] __attribute__ ((aligned (4))) = {\n",argv[1],i,my,mx);
		for(j=0;j<my;j++)
		{
			printf("{ ");
			for(k=0;k<mx;k++)
			{
				if(!(k%16) && k) printf("\n");
				printf(" %d,",(maparray[mx*j+k]?UNOPT_BLOCK_MASK:0)+
					   (blockstr[maparray[mx*j+k]].unused1?UNOPT_WALKABLE_MASK:0)+
					   (blockstr[maparray[mx*j+k]].unused2?UNOPT_INVISIBLE_MASK:0)+
					   (blockstr[maparray[mx*j+k]].trigger?UNOPT_HALFHIGH_MASK:0));
			}
			printf("},\n");
		}
		printf("};\n");
	}
	
	memset(cmapremap, 0, sizeof(cmapremap));
	// ensure color 0 comes first in optimized palette
	palcount=1;
	cmapremap[0]=0;

	for(i=0;i<sizeof(blockgfx);i++)
	{
		remap=1;
		for(j=0;j<palcount;j++)
		{
			if(cmapremap[j]==blockgfx[i])
			{
				remap=0;
				break;
			}
		}
		if(remap)
			cmapremap[palcount++]=blockgfx[i];
	}
	
	printf("const u16 %s_mapcmap[%d] __attribute__ ((aligned (4))) = {\n",argv[1], palcount);
	for(i=0;i< palcount;i++)
	{
		if(i && !(i%8)) printf("\n");
		printf("0x%04x, ",mapcmap[cmapremap[i]]);
	}
	printf("};\n");

	printf("const u8 %s_blockgfx[%d] __attribute__ ((aligned (4))) = {\n",argv[1],sizeof(blockgfx));
	for(i=0;i<sizeof(blockgfx);i++)
	{
		if(i && !(i%8)) printf("\n");
		for(j=0;j<palcount;j++)
		{
			if(cmapremap[j]==blockgfx[i]) break;
		}
		if(j==palcount) exit(5);	// not found
		printf("0x%02x, ", j);
	}
	printf("};\n");

	printf("static const u16* %s_maps[] = {", argv[1]);
	for(i=0; i<atoi(argv[2]); i++) printf(" %s_maparray%d, ", argv[1], i);
	printf("};\n");
	
	printf("static const u8* %s_mapsunopt[] = {",argv[1]);
	for(i=0; i<atoi(argv[2]); i++) printf(" (const u8*)%s_maparray%d_plain, ", argv[1], i);
	printf("};\n");
	
	printf("static const mapdata %s_map = { %d, %s_maps, %s_mapsunopt, %s_blockstr, %s_mapcmap, %d, %s_blockgfx, %d, %d, %d };\n",
		   argv[1], atoi(argv[2]), argv[1], argv[1], argv[1], argv[1], palcount, argv[1], sizeof(blockgfx)/2, mx, my);


	return 0;
}

