// (C) 2003 Ulrich Hecht <uli@emulinks.de>.
// Distributed under the terms of the GNU Public License v2.

#include <mygba.h>

#include <widget.h>

#include <misc.h>
#include <stage.h>

#include <vbl.h>
#include <input.h>

const fontdata* widget::m_characters=0;
const palettedata* widget::m_palette=0;

//#define DEBUG_WIDGETCTOR
//#define DEBUG_SETMAPTILE

widget::widget(int bg, const spritedata* border, int numaddontiles, int xsize, int ysize, int xpos, int ypos, int textdelay, bool transparent)
{
	int i,j;

	m_bg=bg;
	m_xsize=xsize;
	m_ysize=ysize;
	m_xpos=xpos;
	m_ypos=ypos;
	m_textdelay=textdelay;

	m_havebgimage=false;

	m_border=border;
	if (m_border)
	{
		ASSERT(border->xsize==8 && border->ysize==8 && border->animations==8, "invalid border sprite");
		m_borderwidth=1;
	}
	else
		m_borderwidth=0;

	m_movedeltax=m_movedeltay=m_movecountdown=0;
	m_ismoving=false;

	ASSERT(m_characters, "no charset set");
	ASSERT(m_palette, "palette not set");

	// CAVEAT: cbb_only_mode turned off, always use spam_SetMapTileEx()!
#ifdef DEBUG_WIDGETCTOR
	uprintf("widget const %d\n", numaddontiles+t_startofaddongfx);
#endif
	ham_bg[bg].ti=ham_InitTileEmptySet(numaddontiles+t_startofaddongfx, 0, 0);
#ifdef DEBUG_SETMAPTILE
	uprintf("fb %d cbb %d ocbb %d\n", ham_bg[bg].ti->first_block, ham_bg[bg].ti->cbb, ham_bg[bg].ti->offset_from_cbb);
#endif

	ham_bg[bg].mi=ham_InitMapEmptySet(0,0);
#ifdef DEBUG_SETMAPTILE
	uprintf("slotptr %x msize %d\n", (int)MEM_HAMBG_SLOT_PTR(ham_bg[bg].ti->first_block),ham_bg[bg].mi->map_size);
#endif

	// init map to transparent tile; ham_InitMapEmptySet() should do this, but it fails occasionally
	for (i=0;i<32;i++)
		for (j=0;j<32;j++)
			spam_SetMapTileEx(bg, i, j, 0, 0, 0, 15);

	ham_LoadBGPal16((void*)(m_palette->palette), 15);

	// load tiles
	ham_ReloadTileGfx(ham_bg[bg].ti, (u16*)transparent_tile, 0, 1);
	
	// uncompress/load charset
	u16* cs=new u16[W_TOTAL_NUM_CHAR_TILES*4*8/2];
	bios_lz77uncompwram((u16*)(m_characters->font), cs);
	ham_ReloadTileGfx(ham_bg[bg].ti, cs, 1, W_NUM_CHAR_TILES);
	delete[] cs;

	if (m_border) ham_ReloadTileGfx(ham_bg[bg].ti, (u16*)(m_border->graphics), W_NUM_CHAR_TILES+1, t_startofaddongfx-(W_NUM_CHAR_TILES+1));

	clear();
	
	if(transparent)
	{
		ham_SetFxMode(1<<m_bg,0xf&(~(1<<m_bg)),FX_MODE_ALPHABLEND);
		ham_SetFxAlphaLevel(15, 4);
	}
	
	ham_SetBgXY(bg, -xpos, -ypos);
	ham_InitBg(bg, 1, 0, 0);

	vbl::instance()->addhandler(this);
}

void widget::clear()
{
	int i,j;
	if (m_border)
	{
		// draw border

		// corners
		// ham_SetMapTileEx() fails in 16-color modes with cbb_only_mode turned off. Using my own implementation instead.
		spam_SetMapTileEx(m_bg, 0, 0, t_tl,0,0,15);
		spam_SetMapTileEx(m_bg, m_xsize-1, 0, t_tr,0,0,15);
		spam_SetMapTileEx(m_bg, 0, m_ysize-1, t_bl,0,0,15);
		spam_SetMapTileEx(m_bg, m_xsize-1, m_ysize-1, t_br,0,0,15);

		// edges
		for (i=1;i<m_xsize-1;i++)
		{
			spam_SetMapTileEx(m_bg, i, 0, t_top,0,0,15);
			spam_SetMapTileEx(m_bg, i, m_ysize-1, t_bottom,0,0,15);
			for (j=1;j<m_ysize-1;j++)		 // initialise widget body
				spam_SetMapTileEx(m_bg, i, j, t_empty,0,0,15);
		}
		for (i=1;i<m_ysize-1;i++)
		{
			spam_SetMapTileEx(m_bg, m_xsize-1, i, t_right,0,0,15);
			spam_SetMapTileEx(m_bg, 0, i, t_left,0,0,15);
		}
	}
	else
	{
		for(i=0;i<m_xsize;i++)
			for(j=0;j<m_ysize;j++)
				spam_SetMapTileEx(m_bg, i, j, t_empty, 0, 0, 15);
	}
}

void widget::setpos(int x, int y)
{
	m_xpos=x;
	m_ypos=y;
	ham_SetBgXY(m_bg, -x, -y);
}

// assumes a 16 characters wide bitmap layout, starting with SPACE (ASCII 32)
void widget::drawasciichar(int x, int y, unsigned char c)
{
	int offset=c-W_CHAR_ASCII_OFFSET;
	offset=(offset/W_CHAR_BITMAP_WIDTH)*2*W_CHAR_BITMAP_WIDTH+(offset&(W_CHAR_BITMAP_WIDTH-1));
	spam_SetMapTileEx(m_bg, x, y, 1+offset, 0, 0, 15);
	spam_SetMapTileEx(m_bg, x, y+1, 1+W_CHAR_BITMAP_WIDTH+offset, 0, 0, 15);
	if (m_textdelay) for (int i=m_textdelay;i>0;i-=VBL_FRAMESKIP) { if(!input::action()) VBL_NEXT_FRAME; }
}

void widget::drawstring(int x, int y, const char* string)
{
	ASSERT(x+(int)strlen(string)<=m_xsize, "string too wide");
	while (*string)
		drawasciichar(x++, y, *string++);
}

void widget::drawstringn(int x, int y, const char* string, int num)
{
	ASSERT(x+num<=m_xsize, "string too wide");
	for (int i=0;i<num;i++)
		drawasciichar(x++, y, *string++);
}

widget::~widget()
{
	ham_DeInitBg(m_bg);
	if(m_havebgimage)
		ham_DeInitBg(m_bg+1);

	vbl::instance()->removehandler(this);
}

void widget::setbackgroundimage(const backdropdata* backdrop)
{
	if(backdrop)
	{
	m_havebgimage=true;

	ham_LoadBGPal((void*)(backdrop->bgpal),256-16);

	// uncompress tiles
	ham_bg[m_bg+1].ti=ham_InitTileEmptySet(backdrop->bgtiles_size/64, 1, 1);
	bios_lz77uncompvram((u16*)(backdrop->bgtiles), MEM_HAMBG_SLOT_PTR(ham_bg[m_bg+1].ti->first_block));
	
	// uncompress map
	ham_bg[m_bg+1].mi=ham_InitMapEmptySet(0,0);
	u16* bm=new u16[(SCREEN_WIDTH/8)*(SCREEN_HEIGHT/8)];
	bios_lz77uncompwram((u16*)(backdrop->bgmap), bm);
	ham_InsertMapFragment(ham_InitMapFragment((void*)bm, SCREEN_WIDTH/8, SCREEN_HEIGHT/8, 0, 0, SCREEN_WIDTH/8, SCREEN_HEIGHT/8, 0),m_bg+1,0,0);
	delete[] bm;
	
	ham_InitBg(m_bg+1, 1, 3, 0);
	ham_SetBgXY(m_bg+1, 0, 0);
	}
}

void widget::vbl_handler()
{
	if(m_ismoving) moveme();
}
